package com.dragoninfo.dcuc.authweb.interceptor;

import cn.hutool.core.util.StrUtil;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.auth.auth.facade.IBimBusinessFacade;
import com.dragoninfo.dcuc.auth.sub.dto.AuthUserDTO;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthUserInfoFacade;
import com.dragoninfo.dcuc.authweb.config.DcucAuthWebConfig;
import com.dragoninfo.dcuc.authweb.enums.CheckTypeEnum;
import com.dragoninfo.dcuc.authweb.exception.EvidenceException;
import com.dragoninfo.dcuc.authweb.util.HeadTokenUtils;
import com.dragoninfo.dcuc.common.Constants;
import com.dragonsoft.duceap.base.entity.http.HttpStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 拦截器
 *
 * @author yica huangzqa
 * @date 2021/1/18 16:37
 **/
@Slf4j
@Component
public class AuthBeforeResInterceptor implements HandlerInterceptor {

    @Autowired
    private IAuthUserInfoFacade userInfoFacade;

    @Autowired
    private IApplyInfoFacade applyInfoFacade;

    @Autowired
    private IBimBusinessFacade bimBusinessFacade;

    @Autowired
    private DcucAuthWebConfig dcucAuthWebConfig;

    /**
     * 拦截器
     *
     * @param request  HttpServletRequest
     * @param response HttpServletResponse
     * @param handler  Object
     * @return boolean
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {

        CheckTypeEnum checkTypeEnum = dcucAuthWebConfig.getCheckTypeEnum();

        log.info("AuthBeforeResInterceptor自定义拦截器");

        if (checkTypeEnum.equals(CheckTypeEnum.BASE)) {
            String idcard = HeadTokenUtils.getIdcard(request);
            String appCode = HeadTokenUtils.getAppCode(request);

            log.info("Request header idcard:{},appCode:{}", idcard, appCode);

            if (StrUtil.isNotBlank(idcard) && StrUtil.isNotBlank(appCode)) {
                if (StrUtil.isBlank(idcard) || StrUtil.isBlank(appCode)) {
                    throw new EvidenceException(HttpStatus.MULTIPLE_CHOICES.value(), "header参数不能为空");
                }

                AuthUserDTO userInfo = userInfoFacade.findByIdcard(idcard);
                if (userInfo == null) {
                    throw new EvidenceException(HttpStatus.MULTIPLE_CHOICES.value(), "header参数：idcard无权限，或身份证错误");
                }
                //校验应用
                String appId = applyInfoFacade.codeConvertToId(appCode);
                if (StrUtil.isBlank(appId)) {
                    throw new EvidenceException(HttpStatus.MULTIPLE_CHOICES.value(), "header参数：appCode无权限，或appCode错误");
                }

                return true;
            }
            throw new EvidenceException(HttpStatus.MULTIPLE_CHOICES.value(), "基础凭据校验失败");

        } else if (checkTypeEnum.equals(CheckTypeEnum.TOKEN)) {

            String userToken = HeadTokenUtils.getUserToken(request);
            String appToken = HeadTokenUtils.getAppToken(request);
            if (StrUtil.isNotBlank(userToken) && StrUtil.isNotBlank(appToken)) {
                log.info("Request header userToken:{},appToken:{}", userToken, appToken);

                if (StrUtil.isBlank(appToken) || StrUtil.isBlank(userToken)) {
                    throw new EvidenceException(HttpStatus.MULTIPLE_CHOICES.value(), "header参数不能为空");
                }

                return bimBusinessFacade.checkToken(userToken, appToken);
            }

            throw new EvidenceException(HttpStatus.MULTIPLE_CHOICES.value(), "令牌凭据校验失败");

        } else if (checkTypeEnum.equals(CheckTypeEnum.NONE)) {
            return true;
        }

        throw new EvidenceException(HttpStatus.MULTIPLE_CHOICES.value(), "请传入凭据");
    }
}
