package com.dragoninfo.dcuc.authweb.restcontroller.admin;

import com.dragoninfo.dcuc.auth.admin.facade.IAppMtAuthFacade;
import com.dragoninfo.dcuc.auth.admin.facade.IManageInfoFacade;
import com.dragoninfo.dcuc.auth.admin.facade.ITempMtAuthFacade;
import com.dragoninfo.dcuc.auth.admin.vo.AdminAuditVO;
import com.dragoninfo.dcuc.auth.sub.dto.AdminMgeDto;
import com.dragoninfo.dcuc.auth.sub.dto.AuthUserDTO;
import com.dragoninfo.dcuc.authweb.common.SysConstants;
import com.dragoninfo.dcuc.authweb.restcontroller.admin.vo.AdminVo;
import com.dragoninfo.dcuc.authweb.util.VersionUtils;
import com.dragoninfo.dcuc.duceap.enums.UserTypeEnum;
import com.dragoninfo.dcuc.duceap.facade.ICodeListResourceFacade;
import com.dragoninfo.duceap.core.enums.ResultEnum;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.metadata.CodeRecord;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.core.search.filter.Condition;
import com.dragonsoft.duceap.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 管理员Controller
 */
@Api(tags = {"管理员管理接口"})
@RestController
@RequestMapping(value = "/usersvr/"+ VersionUtils.VERSION_UID)
public class AdminManagerController extends BaseController {

    @Autowired
    private IManageInfoFacade iManageInfoFacade;
    @Autowired
    private ITempMtAuthFacade tempMtAuthFacade;

    @Autowired
    private IAppMtAuthFacade iAppMtAuthFacade;

    @Autowired
    private ICodeListResourceFacade codeListResourceFacade;

    /**
     * 获取管理范围内的管理员列表
     *
     * @param searchDTO
     * @return
     */
    @ApiOperation(value = "获取管理员列表")
    @ApiImplicitParam(name = "searchDTO", value = "查询条件")
    @PostMapping(value = "admins")
    public Result<List<AuthUserDTO>> adminList(SearchDTO searchDTO) {
        Page<AuthUserDTO> userInfoPage = iManageInfoFacade.rightRangeManagerList(searchDTO);
        return Result.success(userInfoPage.getTotalElements(), userInfoPage.getContent());
    }

    /**
     * 授权管理员审计
     *
     * @param searchDTO
     * @return
     */
    @ApiOperation(value = "授权管理员审计")
    @ApiImplicitParam(name = "searchDTO", value = "查询条件")
    @PostMapping(value = "admins/audit/_search")
    public Result<List<AdminAuditVO>> adminAuditList(SearchDTO searchDTO) {
        Searchable searchable = Searchable.toSearchable(searchDTO);
        Page<AdminAuditVO> userInfoPage = iManageInfoFacade.adminAuditPage(searchable.toSearchDTO());
        Map<String,String> policeUserTypeList = getCodes( "DM_MAN_TYPE");
        userInfoPage.forEach(item-> item.setManType(policeUserTypeList.get(item.getManType())));
        return Result.success(userInfoPage.getTotalElements(), userInfoPage.getContent());
    }


    private Map<String,String> getCodes(String codeName){
        List<CodeRecord> codeRecordList = codeListResourceFacade.listCode("code", codeName);
        Map<String,String> map=new HashMap<>();
        codeRecordList.forEach(item->{
            map.put(item.getValue(),item.getLabel());
        });
        return map;
    }

    /**
     * 获取非管理员的警员
     *
     * @param searchDTO
     * @return
     */

    @ApiOperation(value = "获取非管理员的警员 ")
    @ApiImplicitParam(name = "searchDTO", value = "查询条件")
    @PostMapping(value = "admin/unAdminPolices")
    public Result<List<AuthUserDTO>> unAdminPolices(SearchDTO searchDTO) {
        Searchable searchable = Searchable.toSearchable(searchDTO);
        Page<AuthUserDTO> userInfoPage = iManageInfoFacade.getUserNotInManage(searchable.toSearchDTO());
        return Result.success(userInfoPage.getTotalElements(), userInfoPage.getContent());
    }


    /**
     * 添加管理员
     *
     * @param map
     * @return
     */
    @ApiOperation(value = "添加管理员")
    @ApiImplicitParam(name = "userIds", value = "用户ids")
    @PostMapping(value = "admin")
    public Result addAdmin(@RequestBody Map map) {
        try {
            String userIds = (String) map.get("userIds");
            ResponseStatus responseStatus = tempMtAuthFacade.checkAuth(userIds);
            if (responseStatus.getStatusCode().equals(ResponseStatus.FAIL_CODE)) {
                return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), responseStatus.getMessage());
            }
            iManageInfoFacade.saveAdmin(userIds);
            return Result.success("保存成功");
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "保存异常");
        }
    }

    /**
     * 删除管理员
     *
     * @param userId
     * @return
     */
    @ApiOperation(value = "删除管理员")
    @ApiImplicitParam(name = "userId", value = "用户id")
    @DeleteMapping(value = "admin/{userId}")
    public Result deleteAdmin(@PathVariable("userId") String userId) {
        try {
            iManageInfoFacade.deleteAdmin(userId);
            return Result.success("删除成功");
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "删除异常");
        }

    }

    /***
     * 判断是否有用户的管理权限
     * @param userId
     * @return*/
    @ApiOperation(value = "判断是否有用户的管理权限")
    @ApiImplicitParam(name = "userId", value = "用户id", required = true)
    @GetMapping(value = "admin/manage/auth")
    public Result manageRangePage(@RequestParam("userId") String userId) {
        boolean flag = iManageInfoFacade.haveAuthWithManage(userId);
        if (flag) {
            return Result.success();
        } else {
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "系统异常");
        }
    }

    /**
     * 管理机构范围保存
     *
     * @return
     */
    @ApiOperation(value = "管理员管理-管理员管理范围保存")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgIds", value = "(orgId:status) status: -1:子节点未展开或无子节点,0:未勾选子节点,1:勾选部分子节点,2:勾选全部子节点", required = true),
            @ApiImplicitParam(name = "userId", value = "用户id", required = true),
            @ApiImplicitParam(name = "type", value = " “”（默认为空）：修改管理员范围时 ; “TEMP”：新增管理员范围时", required = true)
    })
    @PostMapping(value = "admin/mgeOrg")
    public Result mgeOrgSave(@RequestBody AdminVo adminVo) {
        String orgIds = adminVo.getOrgIds();
        String userId = adminVo.getUserId();
        String type = adminVo.getType();
        try {
            if (SysConstants.MT_TEMP.equals(type)) {
                ResponseStatus responseStatus = tempMtAuthFacade.save(orgIds, userId);
            } else {
                iAppMtAuthFacade.save(orgIds, userId);
            }
            return Result.success("保存成功");
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "保存异常");
        }
    }

    /**
     * 管理应用范围保存
     * @return
     */
    @ApiOperation(value = "管理应用范围保存")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgIds", value = "(orgId:status) status: -1:子节点未展开或无子节点,0:未勾选子节点,1:勾选部分子节点,2:勾选全部子节点", required = true),
            @ApiImplicitParam(name = "userId", value = "用户id", required = true)
    })
    @PostMapping(value = "admin/mgeApp")
    public Result mgeAppSave(@RequestBody Map<String,String> map) {
        String orgIds = map.get("orgIds");
        String userId =  map.get("userId");
        try {
            ResponseStatus responseStatus = iAppMtAuthFacade.save(orgIds, userId);
            if(ResponseStatus.FAIL_CODE.equals(responseStatus.getStatusCode())){
                return Result.fail(ResponseStatus.FAIL_CODE,responseStatus.getMessage());
            }
            return Result.success("保存成功");
        }catch (Exception e) {
            logger.error(e.getMessage(),e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(),"保存异常");
        }
    }


    /*   *
     * 本系统菜单权限范围列表
     * @param userId
     * @return
     */
//    @ApiOperation(value = "管理员管理-菜单权限列表")
//    @ApiImplicitParam(name = "userId", value = "用户id", required = true)
//    @PostMapping(value = "admin/meun/range")
//    public Result<List<String>> mgeMenuRightRangeList(@RequestBody AdminVo adminVo) {
//        String userId = adminVo.getUserId();
//        String type = adminVo.getType();
//        List<String> mgeMenuList = new ArrayList<String>();
//        if (SysConstants.MT_TEMP.equals(type)) {
//            mgeMenuList = iTempMenuAuthFacade.mgeMenuRightRangeList(userId);
//            if (mgeMenuList.size() == 0) {
//                for (InitMenuEnum menuEnum : InitMenuEnum.values()) {
//                    mgeMenuList.add(menuEnum.getValue());
//                }
//            }
//        } else {
//            mgeMenuList = iMenuMtAuthFacade.mgeMenuRightRangeList(userId);
//        }
//        return Result.success(mgeMenuList);
//    }


//    /**
//     * 本系统菜单权限范围保存
//     *
//     * @return
//     */
//    @ApiOperation(value = "管理员管理-菜单权限保存")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "menuIds", value = "选中菜单集合的字符串集", required = true),
//            @ApiImplicitParam(name = "userId", value = "用户id", required = true),
//            @ApiImplicitParam(name = "type", value = " “”（默认为空）：修改管理员范围时 ; “TEMP”：新增管理员范围时", required = true)
//    })
//    @PostMapping(value = "admin/meun")
//    public Result mgeMenuSave(@RequestBody AdminVo adminVo) {
//        String menuIds = adminVo.getMenuIds();
//        String userId = adminVo.getUserId();
//        String type = adminVo.getType();
//        try {
//            if (SysConstants.MT_TEMP.equals(type)) {
//                iTempMenuAuthFacade.save(menuIds, userId);
//            } else {
//                iMenuMtAuthFacade.save(menuIds, userId);
//            }
//            return Result.success("保存成功");
//        } catch (Exception e) {
//            logger.error(e.getMessage(), e);
//            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "保存异常");
//        }
//
//    }

//    /**
//     * 根据用户获取本系统菜单权限，没有传用户id则默认查询当前登录用户的菜单管理权限
//     *
//     * @param
//     * @return
//     */
//    @GetMapping(value = "admin/allMenu/{userId}")
//    @ApiImplicitParam(name = "userId", value = "用户id", required = true)
//    @ApiOperation(value = "查询当前登录用户的菜单管理权限")
//    public Result<List<Map<String, Object>>> menuRangeOrgList(@PathVariable String userId) {
//        String appId = "00000000000000000000000000000000";
//        List<TreeNodeVo> menuList = iMenuMtAuthFacade.menuFilter(userId, appId);
//        return Result.success(TreeNodeUtils.generateTree(menuList, new SimpleTreeNodeItemResolver("value", "pid")));
//    }

    //    /**
//     * 获取管理员列表
//     *
//     * @param searchDTO
//     * @return
//     */
//    @ApiOperation(value = "获取授权管理员列表")
//    @ApiImplicitParam(name = "searchDTO", value = "查询条件")
//    @PostMapping(value = "auth/admins")
//    public Result<List<AuthUserDTO>> authAdminList(SearchDTO searchDTO) {
//        Searchable searchable = Searchable.toSearchable(searchDTO);
//        //org移除条件
//        Condition orgId_eq = (Condition) searchable.getSearchParams().get("orgId_eq");
//        String orgId = orgId_eq.getValue().toString();
//        searchable.removeSearchFilter("orgId_eq");
//        String type = "OrgMtAuth";
//        AdminMgeDto dto=new AdminMgeDto(type,orgId,searchable.toSearchDTO());
//        searchable.addSearchFilter("orgId", SearchOperator.eq, orgId);
//        Page<AuthUserDTO> userInfoPage = iManageInfoFacade.rightRangeManagerList(dto);
//        return Result.success(userInfoPage.getTotalElements(), userInfoPage.getContent());
//    }

//    /**
//     * 导入管理员信息
//     *
//     * @param fileId
//     * @return
//     **/
//    @ApiOperation(value = "导入管理员信息")
//    @ApiImplicitParam(name = "fileId", value = "导入文件id", required = true)
//    @PostMapping(value = "admin/import")
//    public Result impExcel(@RequestBody String fileId) {
//        try {
//            fileId = JsonUtils.parseObject(fileId).getString("fileId");
//            DocContentDTO fileContent = uploadhandler.getFileContent(fileId);
//            byte[] fileData = fileContent.getFileData();
//            InputStream inputStream = new ByteArrayInputStream(fileData);
//            List<Map<String, String>> list = ImpExcelUtils.loadAdminInfo(inputStream);
//            ImpParamsVo vo = new ImpParamsVo();
//            vo.setMapList(list);
//            vo.setAuthAccessable(true);
//            ResponseStatus responseStatus = iManageInfoFacade.impManger(vo);
//            return Result.success("导入成功");
//        } catch (IOException e) {
//            logger.error(e.getMessage(), e);
//            return Result.fail("300", "导入文件类型错误");
//        } catch (NumberFormatException e) {
//
//            return Result.fail("300", e.getMessage());
//        } catch (Exception e) {
//            logger.error(e.getMessage(), e);
//            return Result.fail("300", "导入失败");
//        }
//    }

//    /**
//     * 导出管理员名单
//     *
//     * @param searchDTO
//     * @param request
//     * @param response
//     **/
//    @ApiOperation(value = "导出管理员名单")
//    @ApiImplicitParam(name = "searchDTO", value = "查询条件")
//    @PostMapping(value = "admin/export")
//    public void exprotAdminList(SearchDTO searchDTO, HttpServletRequest request, HttpServletResponse response) {
//        String orgId = request.getParameter("orgId");
//        searchDTO.setMetadata(this.template());
//        searchDTO.setSize(String.valueOf(SysConstants.EXPORT_NUMBER_MAX));
//        Searchable searchable = Searchable.toSearchable(searchDTO);
//        searchable.addSearchFilter("deleted", SearchOperator.eq, YesNotEnum.NO.getValue());
//        Page<UserInfo> userInfoPage = iManageInfoFacade.getManageInfoList(searchable.toSearchDTO());
//        List<UserInfo> list = userInfoPage.getContent();
//        for (UserInfo userInfo : list) {
//            OrgTreeNode orgTreeNode = orgInfoFacade.getOrgTreeNode(userInfo.getOrgId());
//            userInfo.setOrgName(orgTreeNode.getLabel());
//        }
//        try {
//            ExportExcelUtils.exportAdminExcel("管理员管理名单", list, Searchable.toSearchable(searchDTO).getMetaData(), request, response);
//        } catch (Exception e) {
//            logger.error(e.getMessage(), e);
//            throw new ApplicationException(300, "导出失败", e);
//        }
//    }

//    private String template() {
//        return "[{\"name\":\"name\",\"displyName\":\"姓名\"}," +
//                "{\"name\":\"policeNumber\",\"displyName\":\"警号\"}," +
//                "{\"name\":\"idcard\",\"displyName\":\"身份证\"}," +
//                "{\"name\":\"orgName\",\"displyName\":\"单位名称\"}," +
//                "{\"name\":\"policeCategory\",\"displyName\":\"警种（表码）\"}" +
//                "{\"name\":\"manType\",\"displayType\":\"CODE\",\"codeId\":\"DM_USER_TYPE\",\"displyName\":\"人员类型\"}" +
//                "{\"name\":\"personMobile\",\"displyName\":\"电话\"}," +
//                "]";
//    }

    /***
     * 获取管理员警员
     * @param searchableVo
     * @return
     *//*
    @ApiOperation(value = "获取管理员警员")
    @ApiImplicitParam(name = "searchableVo", value = "查询条件")
    @PostMapping(value = "admin/adminPolices")
    public Result adminAuditListByOrgId(SearchableVo searchableVo) {
        if (StringUtils.isEmpty(searchableVo.getOrgId())) {
            SecurityUser user = (SecurityUser) ContextUtils.getUserInfo();
            UserInfo userInfo = iUserInfoFacade.userDetail(user.getId());
            searchableVo.setOrgId(userInfo.getOrgId());
        }
        SearchDTO searchDTO = searchableVo.getSearchRequest().toSearchDTO();
        Page<AdminAuditVO> userInfoPage = iManageInfoFacade.adminAuditListByOrgId(searchableVo.getOrgId(), searchDTO);
        return Result.success(userInfoPage.getTotalElements(), userInfoPage.getContent());
    }*/

    /***
     * 获取机构管理员数量
     * @param searchableVo
     * @return
     *//*
    @ApiOperation(value = "获取机构管理员数量")
    @ApiImplicitParam(name = "searchableVo", value = "查询条件")
    @PostMapping(value = "admin/orgAdminsCount")
    public Result adminNumsByOrgId(SearchableVo searchableVo) {
        SearchDTO searchDTO = searchableVo.getSearchRequest().toSearchDTO();
        Page<UserInfo> userInfoPage = new PageImpl<UserInfo>();
        int nums = 0;
        if (StringUtils.isNotEmpty(searchableVo.getOrgId())) {
            List<String> orgIdList = new ArrayList<String>();
            orgIdList.add(searchableVo.getOrgId());
            userInfoPage = iManageInfoFacade.getManageInfoByOrgId(orgIdList, searchDTO);
            nums = userInfoPage.getContent().size();
        }
        return Result.success(userInfoPage.getTotalElements(), userInfoPage.getContent());
    }
*/
    /**
     * 菜单列表
     * @param
     * @return
     *//*
    @ApiOperation(value = "菜单列表")
    @ApiImplicitParam(name = "searchable", value = "查询条件")
    @PostMapping(value = "admin/menu")
    public Result menuList(Searchable searchable) {
        SearchDTO searchDTO = searchable.toSearchDTO();
        List<TreeNodeVo> menuList = iMenuMtAuthFacade.menuList(searchDTO);
        return Result.success(menuList);
    }*/

//        /* **
//     * 获取管理范围包括orgid的管理员列表
//     * @return*/
//    @ApiOperation(value = "获取管理范围包括orgid的管理员列表")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "orgId", value = "(orgId:status) status: -1:子节点未展开或无子节点,0:未勾选子节点,1:勾选部分子节点,2:勾选全部子节点", required = true),
//            @ApiImplicitParam(name = "searchDTO", value = "查询条件", required = true),
//            @ApiImplicitParam(name = "type", value = "1:使用机构管理范围,3:使用授权管理", required = true)
//    })
//    @PostMapping(value = "right/list")
//    public Result<List<UserInfo>> rightRangeList(@RequestBody Map<String,Object> map) {
//        String orgId = (String) map.get("orgId");
//        String type =  (String)map.get("type");
//        SearchDTO searchDTO = (SearchDTO)map.get("searchDTO");
//        Page<UserInfo> userInfoPage = new PageImpl<UserInfo>();
//        if (!"[]".equals(searchDTO.getSearchCondition())) {
//            //带条件搜索
//            userInfoPage = iManageInfoFacade.getManageInfoList(searchDTO);
//        } else {
//            //不带条件搜索
//            userInfoPage = iManageInfoFacade.rightRangeManagerList(orgId, type, searchDTO);
//        }
//        return Result.success(userInfoPage.getTotalElements(), userInfoPage.getContent());
//    }


    /**
     * 管理用户范围保存
     * @return
     *//*
    @ApiOperation(value = "管理机构范围保存")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgIds", value = "(orgId:status) status: -1:子节点未展开或无子节点,0:未勾选子节点,1:勾选部分子节点,2:勾选全部子节点", required = true),
            @ApiImplicitParam(name = "userId", value = "用户id", required = true),
            @ApiImplicitParam(name = "type", value = "1:使用机构管理范围,3:使用授权管理", required = true)
    })
    @PostMapping(value = "admin/mgeUser")
    public Result mgeUserSave(@RequestBody Map<String,String> map) {
        String orgIds = map.get("orgIds");
        String userId =  map.get("userId");
        String type =  map.get("type");
        try {
            if ("1".equals(type)) {
                orgIds = mtAuthService.getMtAuth(userId, SysConstants.MT_ORG);
            }
            ResponseStatus responseStatus = iUserMtAuthFacade.save(orgIds, userId);
            return Result.success("保存成功");
        }catch (Exception e) {
            logger.error(e.getMessage(),e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(),"保存异常");
        }
    }*/

    /**
     * 管理应用范围保存
     * @return
     *//*
    @ApiOperation(value = "管理应用范围保存")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgIds", value = "(orgId:status) status: -1:子节点未展开或无子节点,0:未勾选子节点,1:勾选部分子节点,2:勾选全部子节点", required = true),
            @ApiImplicitParam(name = "userId", value = "用户id", required = true)
    })
    @PostMapping(value = "admin/mgeApp")
    public Result mgeAppSave(@RequestBody Map<String,String> map) {
        String orgIds = map.get("orgIds");
        String userId =  map.get("userId");
        try {
            ResponseStatus responseStatus = iAppMtAuthFacade.save(orgIds, userId);
            return Result.success("保存成功");
        }catch (Exception e) {
            logger.error(e.getMessage(),e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(),"保存异常");
        }
    }*/
    /*
     *
     * 广东：管理员临时管理范围保存
     * @return
     */
   /* @ApiOperation(value = "管理员管理-新增-理员应用范围保存")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgIds", value = "(orgId:status) status: -1:子节点未展开或无子节点,0:未勾选子节点,1:勾选部分子节点,2:勾选全部子节点", required = true),
            @ApiImplicitParam(name = "userId", value = "用户id", required = true)
    })
    @PostMapping(value = "admin/mgeTemp")
    public Result mgeTempSave(@RequestBody AdminVo adminVo) {
        String orgIds = adminVo.getOrgId();
        String userId =  adminVo.getUserId();
        try {
            ResponseStatus responseStatus = iAppMtAuthFacade.save(orgIds, userId);
            return Result.success("保存成功");
        }catch (Exception e) {
            logger.error(e.getMessage(),e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(),"保存异常");
        }
    }*/
    /* *//**
     * 管理员管理范围保存
     * @return
     *//*
    @ApiOperation(value = "管理员管理范围保存")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgIds", value = "(orgId:status) status: -1:子节点未展开或无子节点,0:未勾选子节点,1:勾选部分子节点,2:勾选全部子节点", required = true),
            @ApiImplicitParam(name = "userId", value = "用户id", required = true),
            @ApiImplicitParam(name = "type", value = "1:使用机构管理范围,3:使用授权管理", required = true)
    })
    @PostMapping(value = "admin/mgeMge")
    public Result mgeMgeSave(@RequestBody Map<String,String> map) {
        String orgIds = map.get("orgIds");
        String userId =  map.get("userId");
        String type =  map.get("type");
        try {
            if ("1".equals(type)) {
                orgIds = mtAuthService.getMtAuth(userId, SysConstants.MT_ORG);
            }
            ResponseStatus responseStatus = iMgeMtAuthFacade.save(orgIds, userId);
            return Result.success("保存成功");
        }catch (Exception e) {
            logger.error(e.getMessage(),e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(),"保存异常");
        }

    }*/



    /*  *//**
     * 管理用户权限范围列表
     * @param userId
     * @return
     *//*
    @ApiOperation(value = "管理用户权限范围列表")
    @ApiImplicitParam(name = "userId", value = "用户id", required = true)
    @GetMapping(value = "admin/mgeUserRightRange/{userId}")
    public Result mgeUserRightRangeList(@PathVariable("userId") String userId) {
        List<String> mgeUserList = new ArrayList<String>();
        mgeUserList = iUserMtAuthFacade.mgeUserRightRangeList(userId);
        return Result.success(mgeUserList);
    }*/

    /* *//*
     * 管理机构范围列表
     * @param userId
     * @return
     **//*
    @ApiOperation(value = "管理机构范围列表")
    @ApiImplicitParam(name = "userId", value = "用户id", required = true)
    @GetMapping(value = "admin/mgeOrgRightRange/{userId}")
    public Result mgeOrgRightRangeList(@PathVariable("userId") String userId) {
        List<String> mgeOrgList = iOrgMtAuthFacade.mgeOrgRightRangeList(userId);
        return Result.success(mgeOrgList);
    }*/

    /*  *
     * 管理范围配置页面
     * @param userId
     * @param type   type: 1:机构管理,2:用户管理,3:授权管理
     * @return
     */
/*

    @ApiOperation(value = "管理员-新增-管理员管理范围")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", required = true),
            @ApiImplicitParam(name = "type", value = "", required = true)
    })
    @GetMapping(value = "admin/mgeRightRange/{type}/{userId}")
    public String mgeRightRangeEdit(Model model, @PathVariable("type") String type, @PathVariable("userId") String
            userId) {
        String url = "";
        SecurityUser user = (SecurityUser) ContextUtils.getUserInfo();
        if (SysConstants.MT_ORG.equals(type)) {
            url = "adminManage/manageRange/orgMge";
        } else if (SysConstants.MT_USER.equals(type)) {
            url = "adminManage/manageRange/userMge";
        } else if (SysConstants.MT_MGE.equals(type)) {
            url = "adminManage/manageRange/mgeMge";
        } else if (SysConstants.MT_APP.equals(type)) {
            url = "adminManage/manageRange/appMge";
        } else if ("exportPoliceExcel".equals(type)) {
            url = "usermanagement/policemanagement/orgMgeTree"; //选择机构批量导出警员
        } else if ("exportAuxiExcel".equals(type)) {
            url = "usermanagement/auxiliarypolice/orgMgeTree"; //选择机构批量导出警员
        } else if (SysConstants.MT_TEMP.equals(type)) {
            url = "adminManage/manageRange/tempMge";
        }
        if ("exportPoliceExcel".equals(type) || "exportAuxiExcel".equals(type)) {
            type = SysConstants.MT_ORG;
        }
        String mtOrgIds = mtAuthService.getMtAuth(userId, type);//目标者管理范围
        String userOrgIds = mtAuthService.getMtAuth(user.getId(), SysConstants.MT_MGE);//登录者的管理员管理范围
        model.addAttribute("model", userId);
        model.addAttribute("mtOrgIds", mtOrgIds);
        model.addAttribute("userOrgIds", userOrgIds);
        return url;
    }
*/




    /*@RequestMapping(value = "rightRangeListWithQuery/list")
    @ResponseBody
    public ResponseResult rightRangeListWithQuery(@RequestParam("type") String type,
                                                  Searchable searchable) {
        SearchDTO searchDTO = searchable.toSearchDTO();
        Page<UserInfo> userInfoPage = iManageInfoFacade.rightRangeManagerListWithQuery(type, searchDTO);
        return ResponseResult.newInstance(ResponseResult.toConvert(userInfoPage, Searchable.toSearchable(searchDTO)));
    }*/



    /* *//**
     * 导出管理员
     * @param searchDTO
     * @param request
     *//*
    @ApiOperation(value = "导出管理员")
    @ApiImplicitParam(name = "searchDTO", value = "查询条件")
    @PostMapping(value = "admin/exportManageUser")
    public void listExport(SearchDTO searchDTO, HttpServletRequest request, HttpServletResponse response) {
        String orgId = request.getParameter("orgId");
        Page<UserInfo> userInfoPage = new PageImpl<UserInfo>();
        searchDTO.setPage("1");
        searchDTO.setSize("10000");
        if (StringUtils.isNotEmpty(orgId)) {
            OrgTreeNode orgNode = orgInfoFacade.getOrgTreeNode(orgId);
            List<String> orgIds = new ArrayList<String>();
            if (orgNode != null && orgNode.getChildIds().size() > 0)
                orgIds = orgNode.getChildIds();
            orgIds.add(orgId);
            userInfoPage = iManageInfoFacade.getManageInfoByOrgId(orgIds, searchDTO);
        }
        List<UserInfo> list = userInfoPage.getContent();
        try {
            exportExcel("管理员管理名单", list, Searchable.toSearchable(searchDTO).getMetaData(), request, response);
        } catch (IOException e) {
            logger.error(e.getMessage(), e);
            throw new ApplicationException(300, "导出失败", e);
        }
    }*/

}