package com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.auth0.jwt.JWT;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.auth.auth.dto.AppDataSensitiveLevelDTO;
import com.dragoninfo.dcuc.auth.auth.dto.AppFunInfoDTO;
import com.dragoninfo.dcuc.auth.auth.dto.ServiceAuthResultDTO;
import com.dragoninfo.dcuc.auth.auth.facade.IBimBusinessFacade;
import com.dragoninfo.dcuc.auth.auth.facade.IServiceAuthFlowFacade;
import com.dragoninfo.dcuc.auth.auth.facade.IServiceAuthResultFacade;
import com.dragoninfo.dcuc.auth.auth.facade.IStaffAssignAuthInfoFacade;
import com.dragoninfo.dcuc.auth.auth.vo.ApiAppAuthVo;
import com.dragoninfo.dcuc.auth.power.facade.IAppFunInfoFacade;
import com.dragoninfo.dcuc.auth.sub.dto.AuthUserDTO;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthUserInfoFacade;
import com.dragoninfo.dcuc.auth.sub.vo.AuthUserVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.vo.AppAuthRespVO;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.vo.ServiceAuthResultVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.controller.vo.AppFunApiVo;
import com.dragoninfo.dcuc.authweb.util.HeadTokenUtils;
import com.dragoninfo.dcuc.authweb.util.VersionUtils;
import com.dragoninfo.dcuc.common.Constants;
import com.dragoninfo.dcuc.common.entity.ApiResult;
import com.dragoninfo.dcuc.common.entity.ApiSearchReq;
import com.dragoninfo.dcuc.common.utils.SearchableUtil;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.commons.util.ip.IpUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author yica huangzqa
 * @date 2021/2/24 19:11
 **/
@Slf4j
@RestController
@Api(tags = {"权限管理对外开放接口"})
@RequestMapping(value = "/api/auth-service/" + VersionUtils.VERSION_UID_V1 + "/")
public class AuthServiceController {

    @Autowired
    private IServiceAuthFlowFacade serviceAuthFlowFacade;

    @Autowired
    private IServiceAuthResultFacade serviceAuthResultFacade;

    @Autowired
    private IApplyInfoFacade applyInfoFacade;

    @Autowired
    private IStaffAssignAuthInfoFacade staffAssignAuthInfoFacade;

    @Autowired
    private IAuthUserInfoFacade userFacade;

    @Autowired
    private IBimBusinessFacade bimBusinessFacade;

    @Autowired
    private IAppFunInfoFacade iAppFunInfoFacade;

    @GetMapping(value = "apps/authentication")
    @ApiOperation(value = "应用级鉴权")
    @ApiImplicitParams({@ApiImplicitParam(name = "appsAuthenticationReq", value = "应用级鉴权VO")})
    public ApiResult appsAuthentication(HttpServletRequest request) {

        String userToken = request.getHeader(Constants.DCUC_USER_TOKEN);
        String appToken = request.getHeader(Constants.DCUC_APP_TOKEN);

        if (StrUtil.isBlank(userToken)) {
            return ApiResult.setFailMessage("请传入用户令牌");
        }

        boolean checkUserToken = bimBusinessFacade.checkUserToken(userToken);

        if (!checkUserToken) {
            return ApiResult.setFailMessage("用户令牌无效");
        }

        DecodedJWT decode = JWT.decode(userToken);
        String idcard = decode.getClaim("idCard").asString();
        int exp = decode.getClaim("exp").asInt();
        int current = (int) (System.currentTimeMillis() / 1000);

        int expAt = exp - current;

        log.info("UserToken:{} , idcard:{}, expAt:{}", userToken, idcard, expAt);

        if (expAt < 0) {
            return ApiResult.setFailMessage("令牌已失效");
        }

        AuthUserDTO userInfo = userFacade.findByIdcard(idcard);
        if (userInfo == null) {
            return ApiResult.setFailMessage("用户不存在");
        }

        bimBusinessFacade.cacheUserToken(idcard, userToken, expAt);

        AuthUserVo userVo = new AuthUserVo();
        BeanUtil.copyProperties(userInfo, userVo);
        ApiAppAuthVo authVo = ApiAppAuthVo.builder()
                .requestAppCode(HeadTokenUtils.getAppCode(request))
                .userInfo(userVo)
                .appToken(appToken)
                .userToken(userToken)
                .terminalIp(IpUtils.getIp())
                .build();
        List<AppDataSensitiveLevelDTO> appList = staffAssignAuthInfoFacade.apiAppAuth(authVo);

        List<AppAuthRespVO> respVOList = appList.stream().map(AppAuthRespVO::parseDto)
                .collect(Collectors.toList());
        return ApiResult.setSuccessResult(respVOList);
    }

    /**
     * 服务级鉴权
     *
     * @param request
     * @return
     */
    @PostMapping(value = "service/authentication")
    @ApiOperation(value = "服务级鉴权")
    @ApiImplicitParams({@ApiImplicitParam(name = "ServiceAuthenticationReq", value = "服务鉴权vo")})
    public ApiResult authentication(HttpServletRequest request) {
        String idcard = request.getHeader(Constants.IDCARD);
        String appCode = request.getHeader(Constants.APP_CODE);
        String userToken = request.getHeader(Constants.DCUC_USER_TOKEN);
        String appToken = request.getHeader(Constants.DCUC_APP_TOKEN);

        if (StrUtil.isNotBlank(appToken)) {
            DecodedJWT decode = JWT.decode(appToken);
            appCode = decode.getClaim("appCode").asString();
        }

        if (StrUtil.isBlank(appCode)) {
            return ApiResult.setFailMessage(String.format("appCode: %s is blank.", appCode));
        }

        ApplyInfo applyInfo = applyInfoFacade.getAppByCode(appCode);
        if (applyInfo == null || BooleanEnum.TRUE.value.equals(applyInfo.getApplyStatus())) {
            return ApiResult.setFailMessage(String.format("app: %s is not enabled", appCode));
        }

        if (StringUtils.isEmpty(idcard)) {
            DecodedJWT decode = JWT.decode(userToken);
            idcard = decode.getClaim("idCard").asString();
        }

        Object o = serviceAuthFlowFacade.serviceAuthentication(idcard, appCode, userToken, appToken);
        return ApiResult.setSuccessResult(o);
    }


    @ApiOperation(value = "授权服务列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "ApiSearchReq", value = "授权服务列表查询vo")})
    @PostMapping("service/search")
    public ApiResult serviceSearch(@RequestBody ApiSearchReq apiSearchReq) {
        Map<String, SearchOperator[]> requestMap = new HashMap<>();
        requestMap.put("appCode", new SearchOperator[]{SearchOperator.eq});
        requestMap.put("serviceCode", new SearchOperator[]{SearchOperator.eq});
        requestMap.put("serviceStatus", new SearchOperator[]{SearchOperator.ne, SearchOperator.eq, SearchOperator.in});
        Searchable searchable;
        try {
            searchable = SearchableUtil.parseApiSearchReqToSearchable(apiSearchReq, requestMap, 1000);
        } catch (IllegalArgumentException e) {
            String message = e.getMessage();
            return ApiResult.setFailMessage(message);
        }
        Page<ServiceAuthResultDTO> serviceAuthResultDtoList = serviceAuthResultFacade.serviceAuthResultPage(searchable.toSearchDTO());
        List<ServiceAuthResultDTO> content = serviceAuthResultDtoList.getContent();
        List<ServiceAuthResultVo> voList = new ArrayList<>(content.size());

        content.forEach(item -> {
            ServiceAuthResultVo vo = new ServiceAuthResultVo();
            BeanUtils.copyProperties(item, vo);
            vo.setServiceStatus(item.getAuthStatus());
            vo.setPrivilegedTime(item.getCreateTime());
            voList.add(vo);
        });
        return ApiResult.setSuccessPage(serviceAuthResultDtoList.getTotalElements(), voList);
    }

    @ApiOperation(value = "服务授权撤销")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "appCode", value = "应用编码"),
            @ApiImplicitParam(paramType = "path", name = "serviceCode", value = "服务编码")
    })
    @DeleteMapping("service/{appCode}/{serviceCode}")
    public ApiResult delService(@PathVariable String appCode, @PathVariable String serviceCode) {
        try {
            ServiceAuthResultDTO authDto = serviceAuthResultFacade.getDetailByAppAndService(appCode, serviceCode);
            serviceAuthResultFacade.delServiceAuthResult(authDto);
            return ApiResult.setSuccess();
        } catch (Exception e) {
            e.printStackTrace();
            return ApiResult.setFailMessage("服务授权撤销失败");
        }
    }


    /**
     * 提供给服务方查询
     *
     * @param apiSearchReq
     * @return
     */
    //TODO
    // 授权结果查询需要用到app服务,后期需要将连接查询拆分
    // 并且需要添加授权申请详情表,与申请表是一对多的关系。
    @ApiOperation(value = "服务授权结果列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "ApiSearchReq", value = "授权服务列表查询vo")})
    @PostMapping("result/search")
    public ApiResult serviceResultSearch(@RequestBody ApiSearchReq apiSearchReq) {
        Map<String, SearchOperator[]> requestMap = new HashMap<>();
        requestMap.put("appName", new SearchOperator[]{SearchOperator.eq, SearchOperator.like});
        requestMap.put("appOrgName", new SearchOperator[]{SearchOperator.eq, SearchOperator.like});
        requestMap.put("appOrgCode", new SearchOperator[]{SearchOperator.eq});
        requestMap.put("serviceResourceId", new SearchOperator[]{SearchOperator.eq});
        requestMap.put("appManagerOrgName", new SearchOperator[]{SearchOperator.eq, SearchOperator.like});
        requestMap.put("applicantName", new SearchOperator[]{SearchOperator.eq, SearchOperator.like});
        requestMap.put("serviceName", new SearchOperator[]{SearchOperator.eq, SearchOperator.like});
        requestMap.put("serviceManagerOrgCode", new SearchOperator[]{SearchOperator.eq});
        requestMap.put("serviceManagerOrgName", new SearchOperator[]{SearchOperator.eq, SearchOperator.like});
        requestMap.put("serviceStatus", new SearchOperator[]{SearchOperator.ne, SearchOperator.eq, SearchOperator.in});
        requestMap.put("appStatus", new SearchOperator[]{SearchOperator.ne, SearchOperator.eq, SearchOperator.in});
        Searchable searchable;
        try {
            searchable = SearchableUtil.parseApiSearchReqToSearchable(apiSearchReq, requestMap, 1000);
        } catch (IllegalArgumentException e) {
            String message = e.getMessage();
            return ApiResult.setFailMessage(message);
        }
        Page<ServiceAuthResultDTO> serviceAuthResultDTOS = serviceAuthResultFacade.serviceResultPageJoinFlow(searchable.toSearchDTO());
        List<ServiceAuthResultDTO> content = serviceAuthResultDTOS.getContent();
        List<ServiceAuthResultVo> voList = new ArrayList<>(content.size());

        content.forEach(item -> {
            ServiceAuthResultVo vo = new ServiceAuthResultVo();
            BeanUtils.copyProperties(item, vo);
            vo.setServiceStatus(item.getAuthStatus());
            vo.setPrivilegedTime(item.getAuthTime());
            voList.add(vo);
        });
        return ApiResult.setSuccessPage(serviceAuthResultDTOS.getTotalElements(), voList);
    }

    @ApiOperation(value = "查询应用相关的菜单和功能")
    @PostMapping(value = "/app-functions/search")
    public ApiResult appFunctionSearch(@RequestBody ApiSearchReq searchReq) {
        Map<String, SearchOperator[]> requestMap = new HashMap<>();
        requestMap.put("appCode", new SearchOperator[]{SearchOperator.eq, SearchOperator.in});
        requestMap.put("funCode", new SearchOperator[]{SearchOperator.eq, SearchOperator.in});
        Searchable searchable;
        try {
            searchable = SearchableUtil.parseApiSearchReqToSearchable(searchReq, requestMap);
        } catch (IllegalArgumentException e) {
            String message = e.getMessage();
            return ApiResult.setFailMessage(message);
        }
        ResponseDTO<List<AppFunInfoDTO>> status = iAppFunInfoFacade.apiFuncSearch(searchable.toSearchDTO());
        if(!ResponseStatus.SUCCESS_CODE.equals(status.getStatusCode())) {
            return ApiResult.setFailMessage(status.getMessage());
        } else {
            List<AppFunInfoDTO> result = (List<AppFunInfoDTO>) status.getResult();
            List<AppFunApiVo> funApiVos = result.stream().map(item -> {
                AppFunApiVo vo = new AppFunApiVo();
                vo.setAppName(item.getAppName());
                vo.setAppCode(item.getAppCode());
                vo.setFunCode(item.getCode());
                vo.setFunName(item.getName());
                return vo;
            }).collect(Collectors.toList());
            return ApiResult.setSuccessResult(funApiVos);
        }
    }


}
