package com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.controller;

import cn.hutool.core.util.StrUtil;
import com.auth0.jwt.JWT;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.dragoninfo.dcuc.auth.auth.api.IApiDataAuthFacade;
import com.dragoninfo.dcuc.auth.auth.dto.DataItemsCheckDto;
import com.dragoninfo.dcuc.authweb.util.VersionUtils;
import com.dragoninfo.dcuc.common.Constants;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 代码千万行,注释第一行,编码不规范,同事两行泪
 *
 * @author huang(jy)
 * @version 1.0
 * @date 2021/5/7 9:09
 */
@Api(tags = {"数据鉴权"})
@RestController
@RequestMapping(value = "/api/auth-service/" + VersionUtils.VERSION_UID_V1 + "/")
public class DataAuthApiController {

    @Autowired
    private IApiDataAuthFacade apiDataAuthFacade;

    @ApiOperation(value = "查询是否拥有数据项权限接口")
    @ApiImplicitParams({@ApiImplicitParam(name = "DataItemsCheckDto", value = "数据资源对象")})
    @PostMapping(value = "data-auth/data-items/check")
    public ResponseDTO checkDataItems(@RequestBody DataItemsCheckDto checkDto,
                                      @RequestHeader(value = Constants.DCUC_USER_TOKEN, required = false) String headerUserToken,
                                      @RequestHeader(value = Constants.IDCARD, required =  false) String headerIdcard,
                                      @RequestHeader(value = Constants.DCUC_APP_TOKEN, required = false) String dcucAppToken,
                                      @RequestHeader(value = Constants.APP_CODE, required =  false) String appCode) {
        String currentIdcard = null;
        String currentAppCode = null;
        if (StrUtil.isBlank(headerUserToken)) {
            currentIdcard = headerIdcard;
        }else{
            DecodedJWT decode = JWT.decode(headerUserToken);
            currentIdcard = decode.getClaim("idCard").asString();
        }
        if (StringUtils.isBlank(currentIdcard)){
            return  ResponseDTO.fail("当前登录人身份证号不允许为空", (Object) null);
        }

        if (StrUtil.isNotBlank(appCode)) {
            currentAppCode=appCode;
        }else{
            DecodedJWT decode = JWT.decode(dcucAppToken);
            currentAppCode = decode.getClaim("appCode").asString();
        }

        checkDto.setCurrentIdcard(currentIdcard);
        checkDto.setIdcard(currentIdcard);
        checkDto.setCurrentAppCode(currentAppCode);
        return apiDataAuthFacade.dataItemsCheck(checkDto);
    }
}
