package com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.controller;

import com.dragoninfo.dcuc.auth.auth.dto.ServiceAuthFlowDTO;
import com.dragoninfo.dcuc.auth.auth.facade.IServiceAuthFlowFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.vo.AuthFlowRepeatCheckVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.vo.ResourceInfoVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.vo.ServiceAuthFlowAcceptVo;
import com.dragoninfo.dcuc.authweb.util.HeadTokenUtils;
import com.dragoninfo.dcuc.common.entity.ApiResult;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author mazq
 * @date 2021/2/24
 */
@Api(tags = {"服务授权审批工单处理接口"})
@Controller
@RequestMapping(value = "/api/auth-service/v1/workflow/")
public class WorkFlowApiController {

    @Autowired
    private IServiceAuthFlowFacade serviceAuthFlowFacade;

    /**
     * 服务授权工作单保存
     */
    @ApiOperation(value = "服务授权工单保存")
    @ApiImplicitParams({@ApiImplicitParam(name = "serviceAuthFlowVo", value = "服务授权工单保存Vo")})
    @PostMapping("service")
    @ResponseBody
    public ApiResult serviceAuthFlowSave(@RequestBody @Validated ServiceAuthFlowAcceptVo serviceAuthFlowVo, HttpServletRequest request) {
        String idcard = HeadTokenUtils.getIdcard(request);
        String applyAppCode = HeadTokenUtils.getAppCode(request);
        if (StringUtils.isBlank(idcard)) {
            return ApiResult.setFailMessage("用户凭据为空");
        }
        ServiceAuthFlowDTO dto = new ServiceAuthFlowDTO();
        setAppServiceInfo(dto, serviceAuthFlowVo);
        BeanUtils.copyProperties(serviceAuthFlowVo, dto, "serviceCodes", "serviceNames");
        dto.setUser(idcard);
        dto.setApplyAppCode(applyAppCode);
        ResponseDTO responseDTO = serviceAuthFlowFacade.serviceAuthFlowSave(dto);
        String statusCode = responseDTO.getStatusCode();
        if (ResponseStatus.SUCCESS_CODE.equals(statusCode)) {
            return ApiResult.setSuccess();
        } else {
            ApiResult apiResult = ApiResult.setFailMessage(responseDTO.getMessage());
            apiResult.setResult(responseDTO.getResult());
            return apiResult;
        }
    }

    /**
     * 服务授权申请是否存在重复申请
     */
    @ApiOperation(value = "服务授权申请是否存在重复申请")
    @ApiImplicitParams({@ApiImplicitParam(name = "authFlowRepeatCheckVo", value = "服务授权重复校验Vo")})
    @PostMapping("auth-repeat-check")
    @ResponseBody
    public ApiResult authFlowRepeatCheck(@RequestBody AuthFlowRepeatCheckVo authFlowRepeatCheckVo){
        String appCode = authFlowRepeatCheckVo.getAppCode();
        List<String> serviceCodes = authFlowRepeatCheckVo.getServiceCodes();
        ResponseDTO responseDTO = serviceAuthFlowFacade.authApplyCheck(appCode,serviceCodes);
        String statusCode = responseDTO.getStatusCode();
        if (ResponseStatus.SUCCESS_CODE.equals(statusCode)) {
            return ApiResult.setSuccess();
        } else {
            ApiResult apiResult = ApiResult.setFailMessage(responseDTO.getMessage());
            apiResult.setResult(responseDTO.getResult());
            return apiResult;
        }
    }

    /**
     * 服务授权撤销是否存在授权结果
     */
    @ApiOperation(value = "服务授权撤销是否存在授权结果")
    @ApiImplicitParams({@ApiImplicitParam(name = "authFlowRepeatCheckVo", value = "服务授权重复校验Vo")})
    @PostMapping("auth-cancel-check")
    @ResponseBody
    public ApiResult authCancelCheck(@RequestBody AuthFlowRepeatCheckVo authFlowRepeatCheckVo){
        String appCode = authFlowRepeatCheckVo.getAppCode();
        List<String> serviceCodes = authFlowRepeatCheckVo.getServiceCodes();
        ResponseDTO responseDTO = serviceAuthFlowFacade.authCancelCheck(appCode,serviceCodes);
        String statusCode = responseDTO.getStatusCode();
        if (ResponseStatus.SUCCESS_CODE.equals(statusCode)) {
            return ApiResult.setSuccess();
        } else {
            ApiResult apiResult = ApiResult.setFailMessage(responseDTO.getMessage());
            apiResult.setResult(responseDTO.getResult());
            return apiResult;
        }
    }

    /**
     * 设置申请单appCode和serviceCode信息
     */
    private void setAppServiceInfo(ServiceAuthFlowDTO dto, ServiceAuthFlowAcceptVo serviceAuthFlowVo) {
        ResourceInfoVo resourceInfo = serviceAuthFlowVo.getResourceInfo();
        dto.setAppName(resourceInfo.getVisitorName());
        dto.setAppCode(resourceInfo.getVisitorCode());
        dto.setServiceCodes(resourceInfo.getVisitResourceCode());
        dto.setServiceNames(resourceInfo.getVisitResourceName());
    }
    
}
