package com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v2.controller;

import cn.hutool.core.util.StrUtil;
import com.auth0.jwt.JWT;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.dragoninfo.dcuc.auth.auth.api.IApiDataAuthFacade;
import com.dragoninfo.dcuc.auth.auth.dto.data.DataAuthV2ReqDTO;
import com.dragoninfo.dcuc.auth.auth.vo.DataAuthV2ReqVO;
import com.dragoninfo.dcuc.auth.auth.dto.data.DataAuthV2RespDTO;
import com.dragoninfo.dcuc.auth.auth.vo.DataAuthV2RespVO;
import com.dragoninfo.dcuc.auth.api.vo.BusinessRespEnum;
import com.dragoninfo.dcuc.auth.api.vo.ResultRespVO;
import com.dragoninfo.dcuc.authweb.util.HeadTokenUtils;
import com.dragoninfo.dcuc.authweb.util.VersionUtils;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Collections;
import java.util.Optional;
import java.util.Set;

/**
 * 数据鉴权
 *
 * @author huangzqa
 * @version 2.0
 * @date 2022/10/03
 */
@Slf4j
@Api(tags = {"数据鉴权"})
@RestController
@RequestMapping(value = "/api/auth-service/" + VersionUtils.VERSION_UID + "/")
public class DataAuthApiV2Controller {

    @Autowired
    private IApiDataAuthFacade apiDataAuthFacade;

    @ApiOperation(value = "数据鉴权")
    @ApiImplicitParams({@ApiImplicitParam(name = "dataAuthV2ReqVO", value = "数据资源对象")})
    @PostMapping(value = "data-auth/data-items/check")
    public ResultRespVO<DataAuthV2RespVO> checkDataItems(HttpServletRequest request, @RequestBody DataAuthV2ReqVO dataAuthV2ReqVO) {
        String appTokenId = dataAuthV2ReqVO.getAppTokenId();

        if (StrUtil.isBlank(appTokenId)) {
            return ResultRespVO.resultEnumMessage(BusinessRespEnum.TOKEN_FAIL);
        }

        String resourceId = dataAuthV2ReqVO.getResourceId();
        if (StrUtil.isBlank(resourceId)) {
            return ResultRespVO.resultEnumMessage(BusinessRespEnum.PARAM_ERROR);
        }
        // todo 应用令牌中不包含人员身份证号,需要根据应用令牌查询人员身份证号
        String idcard;
        try {
            DecodedJWT decode = JWT.decode(appTokenId);
            idcard = decode.getClaim("idCard").asString();
        } catch (Exception e) {
            log.error("appTokenId parse error.", e);
            return ResultRespVO.resultEnumMessage(BusinessRespEnum.TOKEN_FAIL);
        }
        if (StringUtils.isBlank(idcard)) {
            return ResultRespVO.resultEnumMessage(BusinessRespEnum.TOKEN_FAIL);
        }

        DataAuthV2ReqDTO v2ReqDTO = new DataAuthV2ReqDTO();
        v2ReqDTO.setIdcard(idcard);
        v2ReqDTO.setResourceId(resourceId);
        v2ReqDTO.setAppToken(appTokenId);
        v2ReqDTO.setUserToken(HeadTokenUtils.getUserToken(request));
        v2ReqDTO.setRequestAppCode(HeadTokenUtils.getAppCode(request));

        ResponseDTO<DataAuthV2RespDTO> dto = apiDataAuthFacade.dataItemsCheckV2(v2ReqDTO);
        if (!ResponseUtil.isSuccess(dto)) {
            ResultRespVO<DataAuthV2RespVO> respVO = ResultRespVO.resultEnumMessage(BusinessRespEnum.AUTH_FAIL);
            respVO.setMessage(dto.getMessage());
            return respVO;
        } else {
            DataAuthV2RespDTO respDTO = (DataAuthV2RespDTO) dto.getResult();
            Set<String> set = respDTO.getItemIdentifier();
            String join = String.join(StrUtil.COMMA, Optional.ofNullable(set)
                    .orElse(Collections.emptySet()));
            DataAuthV2RespVO respVO = new DataAuthV2RespVO();
            respVO.setResourceId(resourceId);
            respVO.setItemIdentifier(join);
            return ResultRespVO.success(respVO);
        }
    }

}
