package com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v2.controller;

import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.app.facade.IServiceResourceFacade;
import com.dragoninfo.dcuc.auth.auth.dto.ServiceAuthFlowDTO;
import com.dragoninfo.dcuc.auth.auth.facade.IServiceAuthFlowFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.vo.ResourceInfoVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.vo.ServiceAuthFlowAcceptVo;
import com.dragoninfo.dcuc.common.Constants;
import com.dragoninfo.dcuc.common.entity.ApiResult;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;

/**
 * @author mazq
 * @date 2021/2/24
 */
@Api(tags = {"服务授权审批工单处理接口"})
@Controller
@RequestMapping(value = "/api/auth-service/v2/workflow/")
public class WorkFlowApiV2Controller {

    @Autowired
    private IServiceAuthFlowFacade serviceAuthFlowFacade;

    @Autowired
    private IApplyInfoFacade applyInfoFacade;

    @Autowired
    private IServiceResourceFacade serviceResourceFacade;
   //流程code
    private  final static String BS_CODE="service-auth-apply";
    //名称（名称随便）
    private  final static String BS_NAME="processName";
    //流程业务类型
    private  final static String BS_TYPE="auth-update";
    /**
     * 服务授权工作单保存
     */
    @ApiOperation(value = "服务授权工单保存")
    @ApiImplicitParams({@ApiImplicitParam(name = "serviceAuthFlowVo", value = "服务授权工单保存Vo")})
    @PostMapping("service")
    @ResponseBody
    public ApiResult serviceAuthFlowSave(@RequestBody @Validated ServiceAuthFlowAcceptVo serviceAuthFlowVo, HttpServletRequest request) {
        String idcard = request.getHeader(Constants.IDCARD);
        String tonken = request.getHeader(Constants.DCUC_USER_TOKEN);
        String applyAppCode = request.getHeader(Constants.APP_CODE);
        String user = StringUtils.isNotBlank(idcard) ? idcard : tonken;
        //放入流程标识码
        if (serviceAuthFlowFacade.checkApplicantId(serviceAuthFlowVo.getApplicantId())) {
            return ApiResult.setFailMessage(String.format("申请单号 %s 不是唯一", serviceAuthFlowVo.getApplicantId()));
        }
        serviceAuthFlowVo.setProcessType(BS_TYPE);
        serviceAuthFlowVo.setBusinessCode(BS_CODE);
        ServiceAuthFlowDTO dto = new ServiceAuthFlowDTO();
        setAppServiceInfo(dto, serviceAuthFlowVo);
        dto.setProcessName(BS_NAME);
        BeanUtils.copyProperties(serviceAuthFlowVo, dto, "serviceCodes", "serviceNames");
        dto.setUser(user);
        dto.setApplyAppCode(applyAppCode);
        ResponseDTO responseDTO = serviceAuthFlowFacade.serviceAuthFlowSave(dto);
        String statusCode = responseDTO.getStatusCode();
        if (ResponseStatus.SUCCESS_CODE.equals(statusCode)) {
            return ApiResult.setSuccess();
        } else {
            ApiResult apiResult = ApiResult.setFailMessage(responseDTO.getMessage());
            apiResult.setResult(responseDTO.getResult());
            return apiResult;
        }
    }


    /**
     * 设置申请单appCode和serviceCode信息
     */
    private void setAppServiceInfo(ServiceAuthFlowDTO dto, ServiceAuthFlowAcceptVo serviceAuthFlowVo) {
        ResourceInfoVo resourceInfo = serviceAuthFlowVo.getResourceInfo();
        dto.setAppName(resourceInfo.getVisitorName());
        dto.setAppCode(resourceInfo.getVisitorCode());
        dto.setServiceCodes(resourceInfo.getVisitResourceCode());
        dto.setServiceNames(resourceInfo.getVisitResourceName());
    }

}
