package com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v3.controller;

import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.auth.auth.dto.AppFunInfoDTO;
import com.dragoninfo.dcuc.auth.auth.dto.AppResourcesDto;
import com.dragoninfo.dcuc.auth.auth.dto.RoleApiDto;
import com.dragoninfo.dcuc.auth.auth.facade.IFunAuthResultFacade;
import com.dragoninfo.dcuc.auth.auth.facade.IRoleFacade;
import com.dragoninfo.dcuc.auth.auth.vo.AppFunVO;
import com.dragoninfo.dcuc.auth.power.facade.IAppFunInfoFacade;
import com.dragoninfo.dcuc.auth.sub.dto.AuthUserDTO;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthUserInfoFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v3.vo.AppFunInfoApiVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v3.vo.FunCodeTreeApiVo;
import com.dragoninfo.dcuc.authweb.util.VersionUtils;
import com.dragoninfo.dcuc.authweb.vo.AuthManagerRes;
import com.dragoninfo.dcuc.authweb.vo.AuthResourceVo;
import com.dragoninfo.dcuc.common.entity.ApiResult;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author yica
 * @date 2021/1/18 14:52
 **/
@RestController
@Api(tags = {"权限管理对外开放接口"})
@RequestMapping(value = "/api/auth-service/" + VersionUtils.VERSION_UID_V3 + "/")
public class AuthV3ApiController {
    private Logger logger = LoggerFactory.getLogger(AuthV3ApiController.class);


    @Autowired
    private IAppFunInfoFacade iAppFunInfoFacade;

    @Autowired
    private IRoleFacade roleFacade;
    @Autowired
    private IAuthUserInfoFacade authUserInfoFacade;

    @Autowired
    private IApplyInfoFacade applyInfoFacade;

    @Autowired
    private IFunAuthResultFacade funAuthResultFacade;

    /**
     * 推送资源
     *
     * @param managerRes AuthManagerRes
     * @return ResultMessage
     */
    @ApiOperation(value = "推送资源")
    @ApiImplicitParams({@ApiImplicitParam(name = "AuthManagerRes", value = "资源vo")})
    @PutMapping(value = "functions")
    public ApiResult sendFunctions(@RequestBody AuthManagerRes managerRes) {
        if (StringUtils.isEmpty(managerRes.getAppCode())) {
            return ApiResult.setFailMessage("appCode不能为空");
        }
        AppResourcesDto dto = new AppResourcesDto();
        dto.setAppCode(managerRes.getAppCode());
        dto.setResources(setAppFun(managerRes.getFunctions()));
        try {
            ResponseDTO responseDTO = iAppFunInfoFacade.updateAppFun(dto);
            if ("300".equals(responseDTO.getStatusCode())) {
                return ApiResult.setFailMessage(responseDTO.getMessage());
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return ApiResult.setFailMessage("保存异常");
        }
        return ApiResult.setSuccess();
    }

    /**
     * 删除资源
     *
     * @param appCode      应用code
     * @param functionCode 资源code
     * @return ResultMessage
     */
    @DeleteMapping(value = "functions/{appCode}/{functionCode}")

    @ApiOperation(value = "推送资源")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "appCode", value = "应用code"),
            @ApiImplicitParam(paramType = "path", name = "functionCode", value = "资源code")})
    public ApiResult delFunctions(@PathVariable("functionCode") String functionCode,
                                  @PathVariable("appCode") String appCode) {
        if (StringUtils.isEmpty(functionCode)) {
            return ApiResult.setFailMessage("functionCode不能为空");
        }
        if (StringUtils.isEmpty(appCode)) {
            return ApiResult.setFailMessage("appCode不能为空");
        }
        ResponseStatus responseStatus = iAppFunInfoFacade.delByAppIdAndFunCode(functionCode, appCode);
        if ("300".equals(responseStatus.getStatusCode())) {
            return ApiResult.setFailMessage(responseStatus.getMessage());
        }
        return ApiResult.setSuccess();
    }

    /**
     * 功能级鉴权
     *
     * @param managerRes
     * @return
     */
    @PostMapping(value = "auths/functions")
    @ApiOperation(value = "功能级鉴权")
    @ApiImplicitParams({@ApiImplicitParam(name = "AuthManagerRes", value = "资源vo")})
    public ApiResult getFunctions(@RequestBody AuthManagerRes managerRes) {
        if (StringUtils.isEmpty(managerRes.getAppCode())) {
            return ApiResult.setFailMessage("appCode is not null");
        }
        ApplyInfo applyInfo = applyInfoFacade.getAppByCode(managerRes.getAppCode());
        if (applyInfo == null || BooleanEnum.TRUE.value.equals(applyInfo.getApplyStatus())) {
            return ApiResult.setFailMessage("app is not enabled");
        }
        if (StringUtils.isEmpty(managerRes.getIdcard())) {
            return ApiResult.setFailMessage("idcard is not null");
        }
        AuthUserDTO userInfo = authUserInfoFacade.findByIdcard(managerRes.getIdcard());
        if (userInfo == null) {
            return ApiResult.setFailMessage("idcard无权限，或身份证错误");
        }
        RoleApiDto roleApiDto = new RoleApiDto();
        roleApiDto.setAppCode(managerRes.getAppCode());
        roleApiDto.setUserId(userInfo.getId());
        roleApiDto.setIdcard(managerRes.getIdcard());
        List<AppFunInfoDTO> menus = roleFacade.getMenus(roleApiDto);
        List<AppFunInfoApiVo> vos = Optional.ofNullable(menus)
                .orElse(new ArrayList<>())
                .stream()
                .map(item -> {
                    AppFunInfoApiVo vo = new AppFunInfoApiVo();
                    vo.setName(item.getName());
                    vo.setCode(item.getCode());
                    vo.setPid(item.getParentId());
                    vo.setUrl(item.getUrl());
                    vo.setSort(item.getSeq());
                    return vo;
                }).collect(Collectors.toList());
        List<FunCodeTreeApiVo> mapList = constructFunCodeTree(vos, null);
        return ApiResult.setSuccessResult(mapList);
    }


    /**
     * 组装树
     *
     * @param maps 数据源
     * @param pid  父节点
     * @return
     */
    private List<FunCodeTreeApiVo> constructFunCodeTree (List<AppFunInfoApiVo> maps, String pid) {
        List<FunCodeTreeApiVo> mapList = new ArrayList<>();
        maps.forEach(item -> {
            FunCodeTreeApiVo vo = new FunCodeTreeApiVo();
            if (item.getPid() == null || StringUtils.isEmpty(item.getPid())) {
                vo.setCode(item.getCode());
                vo.setNextNode(getChildren(maps,item.getCode()));
                mapList.add(vo);
            }

        });
        return mapList;
    }

    /**
     * 组装子节点
     *
     * @param maps 数据源
     * @param pid  父节点
     * @return
     */
    private List<FunCodeTreeApiVo> getChildren(List<AppFunInfoApiVo> maps, String pid) {
        List<FunCodeTreeApiVo> mapList = new ArrayList<>();
        maps.forEach(item -> {
            FunCodeTreeApiVo vo = new FunCodeTreeApiVo();
            if (pid.equals(item.getPid())) {
                vo.setCode(item.getCode());
                vo.setNextNode(getChildren(maps, item.getCode()));
                mapList.add(vo);
            }
        });
        return mapList;
    }

    /**
     * 复制AuthResourceVo到AppFunVO
     *
     * @param list List<AuthResourceVo>
     * @return List<AppFunVO>
     */
    private List<AppFunVO> setAppFun(List<AuthResourceVo> list) {
        List<AppFunVO> appFunVOList = new ArrayList<>();
        list.forEach(item -> {
            AppFunVO appFunVO = new AppFunVO();
            BeanUtils.copyProperties(item, appFunVO);

            appFunVO.setParentCode(item.getUpCode());
            appFunVOList.add(appFunVO);
        });
        return appFunVOList;
    }


}
