package com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v3.controller;

import com.auth0.jwt.JWT;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.dragoninfo.dcuc.auth.auth.dto.ActivitiHolderDTO;
import com.dragoninfo.dcuc.auth.auth.dto.ServiceAuthFlowDTO;
import com.dragoninfo.dcuc.auth.auth.enumresources.WorkFlowStatusEnum;
import com.dragoninfo.dcuc.auth.auth.facade.IServiceAuthFlowFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.vo.ActivitiHolderVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.vo.ResourceInfoVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.vo.ServiceAuthFlowAcceptVo;
import com.dragoninfo.dcuc.common.Constants;
import com.dragoninfo.dcuc.common.entity.ApiResult;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.commons.util.collections.CollectionUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

/**
 * @author mazq
 * @date 2021/2/24
 */
@Api(tags = {"服务授权审批工单处理接口"})
@Controller
@RequestMapping(value = "/api/auth-service/v3/workflow/")
public class WorkFlowApiV3Controller {

    @Autowired
    private IServiceAuthFlowFacade serviceAuthFacade;

   //流程code
    private  final static String BS_CODE="service-auth-apply";
    //名称（名称随便）
    private  final static String BS_NAME="权限新增";
    //流程业务类型
    private  final static String BS_TYPE="auth-update";
    //权限移除流程code
    private  final static String BS_REMOVE_CODE="service-auth-remove";
    //权限移除流程业务类型
    private  final static String BS_REMOVE_TYPE="auth-update";
    //权限移除流程业务类型
    private  final static String BS_REMOVE_NAME="权限移除";

    /**
     * 服务授权工作单保存
     */
    @ApiOperation(value = "服务授权工单保存")
    @ApiImplicitParams({@ApiImplicitParam(name = "serviceAuthFlowVo", value = "服务授权工单保存Vo")})
    @PostMapping("service")
    @ResponseBody
    public ApiResult serviceAuthFlowSave(@RequestBody ServiceAuthFlowAcceptVo serviceAuthFlowVo, HttpServletRequest request) {
        String idcard = request.getHeader(Constants.BUS_SRE_IDCARD);
        String appToken = request.getHeader(Constants.BUS_SRA_TOKEN);
        String userToken = request.getHeader(Constants.BUS_SRE_TOKEN);
        String applyAppCode = request.getHeader(Constants.BUS_SRA_ID);
        if(StringUtils.isBlank(idcard)) {
            DecodedJWT decode = JWT.decode(userToken);
            idcard = decode.getClaim("idCard").asString();
        }
        if(StringUtils.isBlank(applyAppCode)) {
            DecodedJWT decode = JWT.decode(appToken);
            applyAppCode = decode.getClaim("appCode").asString();
        }

        if (StringUtils.isEmpty(serviceAuthFlowVo.getFlowType())) {
            return ApiResult.setFailMessage("工作单类型不能为空");
        }
        //放入流程标识码
        if (serviceAuthFacade.checkApplicantId(serviceAuthFlowVo.getApplicantId())) {
            return ApiResult.setFailMessage(String.format("申请单号 %s 不是唯一", serviceAuthFlowVo.getApplicantId()));
        }
        ServiceAuthFlowDTO dto = new ServiceAuthFlowDTO();
        setAppServiceInfo(dto, serviceAuthFlowVo);

        BeanUtils.copyProperties(serviceAuthFlowVo, dto, "serviceCodes", "serviceNames");
        dto.setProcessName(BS_NAME);
        dto.setProcessType(BS_TYPE);
        dto.setBusinessCode(BS_CODE);
        dto.setUser(idcard);
        dto.setUserToken(userToken);
        dto.setAppToken(appToken);
        dto.setApplyAppCode(applyAppCode);
        dto.setOperateType(WorkFlowStatusEnum.APPLYING.getValue());
        dto.setServiceSharedWay(serviceAuthFlowVo.getFwgxfs());
        //任务信息
        if (CollectionUtils.isNotEmpty(serviceAuthFlowVo.getActivitiHolderList())){
            List<ActivitiHolderDTO> activitiHolderDTOS=new ArrayList<>();
            for (ActivitiHolderVo activitiHolderVo : serviceAuthFlowVo.getActivitiHolderList()) {
                ActivitiHolderDTO activitiHolderDTO=new ActivitiHolderDTO();
                BeanUtils.copyProperties(activitiHolderVo,activitiHolderDTO);
                activitiHolderDTOS.add(activitiHolderDTO);
            }
            dto.setActivitiHolderDTOSList(activitiHolderDTOS);
        }
        ResponseDTO responseDTO = serviceAuthFacade.serviceAuthFlowSave(dto);
        String statusCode = responseDTO.getStatusCode();
        if (ResponseStatus.SUCCESS_CODE.equals(statusCode)) {
            return ApiResult.setSuccess();
        } else {
            return ApiResult.setFailMessage(responseDTO.getMessage());
        }
    }


    /**
     * 设置申请单appCode和serviceCode信息
     */
    private void setAppServiceInfo(ServiceAuthFlowDTO dto, ServiceAuthFlowAcceptVo serviceAuthFlowVo) {
        ResourceInfoVo resourceInfo = serviceAuthFlowVo.getResourceInfo();
        dto.setAppName(resourceInfo.getVisitorName());
        dto.setAppCode(resourceInfo.getVisitorCode());
        dto.setServiceCodes(resourceInfo.getVisitResourceCode());
        dto.setServiceNames(resourceInfo.getVisitResourceName());
    }

}
