package com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v4.controller;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.dragoninfo.dcuc.auth.api.enums.zerotrust.ZeroTrustBusinessRespEnum;
import com.dragoninfo.dcuc.auth.api.vo.zerotrust.ZeroTrustDataRespVO;
import com.dragoninfo.dcuc.auth.api.vo.zerotrust.authticate.*;
import com.dragoninfo.dcuc.auth.auth.api.IZeroTrustAuthFacade;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2022/8/24
 */
@Slf4j
@RestController
@Api(tags = {"权限管理对外开放接口V4"})
@RequestMapping(value = "/api/auth-service/v4/")
public class AuthV4Controller {

    @Autowired
    private IZeroTrustAuthFacade zeroTrustAuthFacade;

    /**
     * 应用级鉴权
     *
     * @param appAuthReqVO 应用级鉴权请求
     * @return 应用级权限
     */
    @PostMapping("appAuth")
    public ZeroTrustDataRespVO<String> appAuth(@RequestBody AppAuthReqVO appAuthReqVO) {
        String userTokenId = appAuthReqVO.getUserTokenId();
        if (StrUtil.isBlank(userTokenId)) {
            log.error("鉴权参数为空");
            return ZeroTrustDataRespVO.resultEnumMessage(ZeroTrustBusinessRespEnum.OPERATE_FAIL);
        }
        ZeroTrustDataRespVO<String> respVo = zeroTrustAuthFacade.appAuth(appAuthReqVO);
        log.info("应用鉴权结果:{}, userTokenId:{}", JSON.toJSONString(respVo), userTokenId);
        return respVo;
    }

    /**
     * 功能级鉴权
     *
     * @param functionAuthReqVO 功能级鉴权求
     * @return 功能级鉴权
     */
    @PostMapping("functionAuth")
    public ZeroTrustDataRespVO<String> functionAuth(@RequestBody FunctionAuthReqVO functionAuthReqVO) {
        String appTokenId = functionAuthReqVO.getAppTokenId();
        String taskId = functionAuthReqVO.getTaskId();
        if (StrUtil.isBlank(appTokenId) || StrUtil.isBlank(taskId)) {
            log.error("鉴权参数为空");
            return ZeroTrustDataRespVO.resultEnumMessage(ZeroTrustBusinessRespEnum.OPERATE_FAIL);
        }
        ZeroTrustDataRespVO<String> respVO = zeroTrustAuthFacade.functionAuth(functionAuthReqVO);
        log.info("请求参数appTokenId:{}, taskId:{}, 功能鉴权结果:{}", appTokenId, taskId, JSON.toJSONString(respVO));
        return respVO;
    }

    /**
     * 服务级鉴权
     *
     * @param serviceAuthReqVO 服务级鉴权请求
     * @return 服务级鉴权
     */
    @PostMapping("serviceAuth")
    public ZeroTrustDataRespVO<String> serviceAuth(@RequestBody ServiceAuthReqVO serviceAuthReqVO) {
        String appTokenId = serviceAuthReqVO.getAppTokenId();
        String taskId = serviceAuthReqVO.getTaskId();
        if (StrUtil.isBlank(taskId) || StrUtil.isBlank(appTokenId)) {
            log.error("鉴权参数为空");
            return ZeroTrustDataRespVO.resultEnumMessage(ZeroTrustBusinessRespEnum.OPERATE_FAIL);
        }

        ZeroTrustDataRespVO<String> respVO = zeroTrustAuthFacade.serviceAuth(serviceAuthReqVO);
        log.info("appTokenId:{}, taskId:{}, 服务级鉴权结果:{}", appTokenId, taskId, JSON.toJSONString(respVO));
        return respVO;
    }

    /**
     * 数据级鉴权
     *
     * @param dataAuthReqVO 数据级鉴权
     * @return 数据级鉴权
     */
    @PostMapping("dataAuth")
    public ZeroTrustDataRespVO<DataAuthRespVO> dataAuth(@RequestBody DataAuthReqVO dataAuthReqVO) {
        String resourceId = dataAuthReqVO.getResourceId();
        String taskId = dataAuthReqVO.getTaskId();
        String appTokenId = dataAuthReqVO.getAppTokenId();
        if (StrUtil.isBlank(resourceId) || StrUtil.isBlank(taskId) || StrUtil.isBlank(appTokenId)) {
            log.error("鉴权参数为空");
            return ZeroTrustDataRespVO.resultEnumMessage(ZeroTrustBusinessRespEnum.OPERATE_FAIL, DataAuthRespVO.empty());
        }
        ZeroTrustDataRespVO<DataAuthRespVO> respVO = zeroTrustAuthFacade.dataAuth(dataAuthReqVO);
        log.info("请求参数resourceId:{}, taskId:{}, appTokenId:{}, 数据鉴权结果:{}", resourceId, taskId, appTokenId, JSON.toJSONString(respVO));
        return respVO;
    }

}
