package com.dragoninfo.dcuc.authweb.restcontroller.api.controller;

import com.dragoninfo.dcuc.auth.auth.entity.RoleInfo;
import com.dragoninfo.dcuc.auth.auth.facade.IRoleFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author huangzqa
 */
@Api(tags = {"角色管理接口"})
@RestController
@RequestMapping(value = "/rest/authmanage/")
public class RestAuthController {

    @Autowired
    private IRoleFacade roleFacade;

    @ApiOperation(value = "查询角色信息")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "roleId", value = "角色ID", required = true
            , example = "40288a8b699fc2500169a33b20540000")})
    @GetMapping(value = "v1/role/{roleId}")
    @ResponseBody
    public RoleInfo role(@PathVariable(value = "roleId") String roleId) {
        return roleFacade.detail(roleId);
    }

    @ApiOperation(value = "查询角色列表")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "userId", value = "用户ID", required = true
            , example = "40288a8b699fc2500169a33b20540000")})
    @GetMapping(value = "v1/roles/{userId}")
    @ResponseBody
    public List<RoleInfo> roles(@PathVariable(value = "userId") String userId) {
        return roleFacade.rolesByUserId(userId);
    }

    @ApiOperation(value = "查询用户列表")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "roleId", value = "角色ID", required = true
            , example = "40288a8b699fc2500169a33b20540000")})
    @GetMapping(value = "v1/users/{roleId}")
    @ResponseBody
    public List<String> users(@PathVariable(value = "roleId") String roleId) {
        return roleFacade.getUserIds(roleId);
    }

    @ApiOperation(value = "查询角色列表")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "appId", value = "应用ID", required = true
            , example = "40288a8b699fc2500169a33b20540000")})
    @GetMapping(value = "v1/roles/app/{appId}")
    @ResponseBody
    public List<RoleInfo> rolesbyappid(@PathVariable(value = "appId") String appId) {
        return roleFacade.getRolesByAppId(appId);
    }

    @ApiOperation(value = "查询角色列表")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "userId", value = "用户ID", required = true
            , example = "40288a8b699fc2500169a33b20540000"),
            @ApiImplicitParam(paramType = "path", name = "appCode", value = "应用代码", required = true
                    , example = "40288a8b699fc2500169a33b20540000")})
    @GetMapping(value = "v1/roles/{userId}/{appCode}")
    @ResponseBody
    public List<RoleInfo> rolesByUserIdAndAppCode(@PathVariable(value = "userId") String userId, @PathVariable(value = "appCode") String appCode) {
        return roleFacade.getRolesByUserIdAndAppCode(userId, appCode);
    }


}
