package com.dragoninfo.dcuc.authweb.restcontroller.api.controller;

import com.dragoninfo.dcuc.auth.list.dto.AppFunRedListDTO;
import com.dragoninfo.dcuc.auth.list.dto.WhiteListUserDTO;
import com.dragoninfo.dcuc.auth.list.facade.IRedListFacade;
import com.dragoninfo.dcuc.auth.list.facade.IWhiteListFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.api.controller.vo.AppFunRedListVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.controller.vo.WhiteListUserApiVo;
import com.dragoninfo.dcuc.common.entity.ApiResult;
import com.dragoninfo.dcuc.common.entity.ApiSearchReq;
import com.dragoninfo.dcuc.common.utils.SearchableUtil;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author mazq
 * @date 2021/7/20
 */
@Api(value = "红白名单查询接口")
@RestController
@RequestMapping("/api/list-service/v1/list")
public class WhiteRedListApiController {

    @Autowired
    private IRedListFacade redListFacade;

    @Autowired
    private IWhiteListFacade whiteListFacade;

    @ApiOperation(value = "菜单和功能红名单查询")
    @PostMapping("func-red-list")
    public ApiResult apiFuncRedListSearch(@RequestBody ApiSearchReq apiSearchReq) {
        Map<String, SearchOperator[]> requestMap = new HashMap<>();
        requestMap.put("appCode", new SearchOperator[]{SearchOperator.eq, SearchOperator.in});
        requestMap.put("funCode", new SearchOperator[]{SearchOperator.eq, SearchOperator.in});
        Searchable searchable;
        try {
            searchable = SearchableUtil.parseApiSearchReqToSearchable(apiSearchReq, requestMap);
        } catch (IllegalArgumentException e) {
            String message = e.getMessage();
            return ApiResult.setFailMessage(message);
        }
        ResponseDTO<List<AppFunRedListDTO>> status = redListFacade.apiFunRedListSearch(searchable.toSearchDTO());
        if(!ResponseStatus.SUCCESS_CODE.equals(status.getStatusCode())) {
            return ApiResult.setFailMessage(status.getMessage());
        } else {
            List<AppFunRedListDTO> result = (List<AppFunRedListDTO>) status.getResult();
            List<AppFunRedListVo> collect = result.stream().map(item -> {
                AppFunRedListVo vo = new AppFunRedListVo();
                BeanUtils.copyProperties(item, vo);
                return vo;
            }).collect(Collectors.toList());
            return ApiResult.setSuccessResult(collect);
        }
    }

    @ApiOperation(value = "人员白单查询")
    @PostMapping("user-white-list")
    public ApiResult userWhiteListSearch(@RequestBody ApiSearchReq apiSearchReq) {
        Map<String, SearchOperator[]> requestMap = new HashMap<>();
        requestMap.put("idcard", new SearchOperator[]{SearchOperator.eq, SearchOperator.in});
        Searchable searchable;
        try {
            searchable = SearchableUtil.parseApiSearchReqToSearchable(apiSearchReq, requestMap);
        } catch (IllegalArgumentException e) {
            String message = e.getMessage();
            return ApiResult.setFailMessage(message);
        }
        ResponseDTO<List<WhiteListUserDTO>> status =  whiteListFacade.apiUserWhiteListSearch(searchable.toSearchDTO());
        if(!ResponseStatus.SUCCESS_CODE.equals(status.getStatusCode())) {
            return ApiResult.setFailMessage(status.getMessage());
        } else {
            List<WhiteListUserDTO> result = (List<WhiteListUserDTO>) status.getResult();
            List<WhiteListUserApiVo> collect = result.stream().map(item -> {
                WhiteListUserApiVo vo = new WhiteListUserApiVo();
                BeanUtils.copyProperties(item, vo);
                return vo;
            }).collect(Collectors.toList());
            return ApiResult.setSuccessResult(collect);
        }
    }




}
