package com.dragoninfo.dcuc.authweb.restcontroller.app;

import com.dragoninfo.dcuc.auth.auth.dto.ServiceAuthFlowDTO;
import com.dragoninfo.dcuc.auth.auth.facade.IServiceAuthFlowFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.app.vo.ServiceAuthFlowVo;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.core.entity.page.PageImpl;
import com.dragonsoft.duceap.core.entity.page.PageRequest;
import com.dragonsoft.duceap.core.search.Searchable;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 服务授权、鉴权接口
 * @author mazq
 * @date 2021/2/22
 */
@RestController
@RequestMapping(value = "/resource/auth")
public class ResourceAuthController {

    @Autowired
    private IServiceAuthFlowFacade serviceAuthFlowFacade;

    /**
     * 服务授权工单列表
     * @return
     */
    @ApiOperation(value = "查询应用列表")
    @ApiImplicitParam(name = "searchable", value = "查询条件")
    @PostMapping(value = "serviceAuthFlow/search", produces = MediaType.APPLICATION_JSON_VALUE)
    public Result<Page<ServiceAuthFlowVo>> serviceAuthFlowPage(Searchable searchable) {
        Page<ServiceAuthFlowDTO> page = serviceAuthFlowFacade.serviceAuthFlowPage(searchable.toSearchDTO());
        List<ServiceAuthFlowDTO> content = page.getContent();
        List<ServiceAuthFlowVo> vos = new ArrayList<>();
        for (ServiceAuthFlowDTO dto : content) {
            ServiceAuthFlowVo vo = new ServiceAuthFlowVo();
            BeanUtils.copyProperties(dto, vo, "serviceCodes", "serviceNames");
            String serviceCodes = dto.getServiceCodes();
            if (StringUtils.isNotBlank(serviceCodes)) {
                List<String> serviceCodesList = Arrays.asList(serviceCodes.split(","));
                vo.setServiceCodes(serviceCodesList);
            }
            String serviceNames = dto.getServiceNames();
            if (StringUtils.isNotBlank(serviceNames)) {
                List<String> serviceNamesList = Arrays.asList(serviceNames.split(","));
                vo.setServiceNames(serviceNamesList);
            }
            vos.add(vo);
        }
        Pageable newPageable = new PageRequest(page.getNumber(), page.getSize());
        PageImpl<ServiceAuthFlowVo> pageResult = new PageImpl<>(vos, newPageable, page.getTotalElements());
        return Result.success(pageResult);
    }
}
