package com.dragoninfo.dcuc.authweb.restcontroller.app;

import com.dragoninfo.dcuc.app.dto.ResourceRequestParamDTO;
import com.dragoninfo.dcuc.app.dto.ResourceRequestResultDTO;
import com.dragoninfo.dcuc.app.facade.IResourceFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.app.vo.ResourceRequestParam;
import com.dragoninfo.dcuc.authweb.restcontroller.app.vo.ResourceRequestResult;
import com.dragoninfo.duceap.core.response.Result;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author mazq
 * @Description 主动获取应用服务数据的接口
 * @create 2020-11-25 15:07
 */
@RestController
@RequestMapping(value = "/applicationManagement/resource/")
public class ResourceListingController {

    @Autowired
    private IResourceFacade resourceFacade;

    /**
     * 获取应用和服务资源列表
     * 全量获取时,如果是第一页获取失败表示该资源列表都获取失败
     */
    @ApiOperation(value = "获取应用和服务资源列表")
    @ApiImplicitParam(name = "requestParam", value = "获取资源数据请求数据封装对象")
    @PostMapping(value = "getResourceRemote")
    public Result<List<ResourceRequestResult>> getResourceListing(@RequestBody ResourceRequestParam requestParam) {
        ResourceRequestParamDTO paramDTO = new ResourceRequestParamDTO();
        BeanUtils.copyProperties(requestParam,paramDTO);
        List<ResourceRequestResultDTO> results = resourceFacade.getResourceRequestResults(paramDTO);
        List<ResourceRequestResult> list = results.stream().map(item -> {
            ResourceRequestResult vo = new ResourceRequestResult();
            BeanUtils.copyProperties(item, vo);
            return vo;
        }).collect(Collectors.toList());
        return Result.success(list);
    }


    /**
     * 资源同步表与应用、服务资源同步
     */
    @ApiOperation(value = "资源同步表与应用、服务资源同步")
    @GetMapping(value = "sync")
    public Result resourceSync() {
        resourceFacade.resourceSync();
        return Result.success();
    }

}
