package com.dragoninfo.dcuc.authweb.restcontroller.audit;

import com.dragoninfo.dcuc.auth.admin.dto.MgeLogDTO;
import com.dragoninfo.dcuc.auth.admin.enumresource.AdminObjectTypeEnum;
import com.dragoninfo.dcuc.auth.admin.facade.IMgeLogFacade;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthOrgInfoFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.audit.vo.MgeLogVo;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@Api(tags = {"管理员审计报表"})
@RestController
@RequestMapping(value = "/auditsvr/manage/")
public class MgeAuditRptController {
    @Autowired
    private IMgeLogFacade iMgeLogFacade;
    @Autowired
    private IAuthOrgInfoFacade authOrgInfoFacade;

    /**
     * 获取授权范围日志
     * -1:子节点未展开或无子节点,0:未勾选子节点,1:勾选部分子节点,2:勾选全部子节点
     * <p>
     * 操作业务类型   10：新增管理员   11：取消管理员
     * 20：新增平台菜单权限   21：取消平台菜单权限
     * 30：新增管理机构权限   31：取消管理机构权限
     * 40：新增管理人员权限   41：取消管理人员权限
     *
     * @param type   日志类型 10：管理员  20：平台菜单   30：机构管理   40：人员管理 50:授权管理 60:管理员管理
     * @param userId 用户ID
     * @return 授权范围日志
     */
    @ApiOperation(value = "获取授权范围日志")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "日志类型 10：管理员  20：平台菜单   30：机构管理   40：人员管理 50:授权管理 60:管理员管理"),
            @ApiImplicitParam(name = "userId", value = "用户ID"),
    })
    @GetMapping(value = "auditList/{type}/{userId}")
    public Result<List<MgeLogVo>> auditList(@PathVariable("type") String type, @PathVariable("userId") String userId) {
        List<MgeLogDTO> mgeLogList = iMgeLogFacade.queryList(type, userId);
        mgeLogList = formatMgeLog(mgeLogList, "list");
        List<MgeLogVo> vos = new ArrayList<>();
        for (MgeLogDTO mgeLog : mgeLogList) {
            MgeLogVo vo = new MgeLogVo();
            BeanUtils.copyProperties(mgeLog, vo);
            vos.add(vo);
        }
        return Result.success(vos);
    }

    /**
     * 格式化审计Log
     *
     * @param mgeLogList
     * @return
     */
    private List<MgeLogDTO> formatMgeLog(List<MgeLogDTO> mgeLogList, String listOrRpt) {
        for (MgeLogDTO mgeLog : mgeLogList) {
            String type = mgeLog.getObjectType();//操作对象类型
            String ids = mgeLog.getObjectId();
            ids = StringUtils.isEmpty(ids) ? "" : ids;
            if (!AdminObjectTypeEnum.GLY.getValue().equals(type) && !AdminObjectTypeEnum.PTCD.getValue().equals(type)) {
                if (AdminObjectTypeEnum.JGGL.getValue().equals(type) || AdminObjectTypeEnum.RYGL.getValue().equals(type) || AdminObjectTypeEnum.SQGL.getValue().equals(type) || AdminObjectTypeEnum.GLYGL.getValue().equals(type)) {
                    ids = authOrgInfoFacade.changeIdsToNames(ids);
                    if(StringUtils.isNotBlank(ids)) {
                        ids = ids.replace(":-1", "（及其下属机构）").replace(":0", "").replace(":1", "").replace(":2", "（及其下属机构）");
                    }
                    mgeLog.setObjectName(ids);
                }

            }
            if (StringUtils.isNotEmpty(mgeLog.getOperateOrgId())) {
                String operateOrgName = authOrgInfoFacade.changeIdsToNames(mgeLog.getOperateOrgId().trim());
                mgeLog.setOperateOrgName(operateOrgName);
            }
        }
        return mgeLogList;
    }
}
