package com.dragoninfo.dcuc.authweb.restcontroller.audit.v2;

import com.dragoninfo.dcuc.auth.audit.facade.IAuthenticationLogFacade;
import com.dragoninfo.dcuc.auth.audit.vo.AuthenticationLogVo;
import com.dragoninfo.dcuc.auth.audit.vo.AuthenticationStatisticsVo;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author: qiuyu
 * @Date: 2021/5/25 14:31
 * @Description:
 */
@Api(tags = {"服务鉴权日志"})
@RestController
@RequestMapping(value = "auditsvr/v2/authenticationLog")
public class AuthenticationLogController {

    public static Logger logger = LoggerFactory.getLogger(AuthenticationLogController.class);

    @Autowired
    private IAuthenticationLogFacade authenticationLogFacade;

    @ApiOperation(value = "日志列表")
    @ApiImplicitParam(name = "searchDTO", value = "查询条件")
    @PostMapping(value = "_search")
    public Result<List<AuthenticationLogVo>> paging(SearchDTO searchDTO) {
        Page<AuthenticationLogVo> page = authenticationLogFacade.paging(searchDTO);
        return Result.success(page.getTotalElements(), page.getContent());
    }

    @ApiOperation(value = "日志详情")
    @ApiImplicitParam(name = "id", value = "方案id")
    @GetMapping(value = "/{id}")
    public Result<AuthenticationLogVo> findOne(@PathVariable(value = "id") Long id) {
        return Result.success(authenticationLogFacade.findOne(id));
    }

    @ApiOperation(value = "统计信息")
    @ApiImplicitParam(name = "searchDTO", value = "查询条件")
    @PostMapping(value = "statistics")
    public Result<List<AuthenticationStatisticsVo>> statistics(SearchDTO searchDTO) {
        Page<AuthenticationStatisticsVo> statistics = authenticationLogFacade.statistics(searchDTO);
        return Result.success(statistics.getTotalElements(), statistics.getContent());
    }


}
