package com.dragoninfo.dcuc.authweb.restcontroller.audit.v2;

import com.dragoninfo.dcuc.auth.audit.dto.RiskProgrammeDto;
import com.dragoninfo.dcuc.auth.audit.facade.IRiskProgrammeFacade;
import com.dragoninfo.dcuc.auth.audit.vo.RiskProgrammeVo;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * @Author: qiuyu
 * @Date: 2021/5/25 14:31
 * @Description:
 */
@Api(tags = {"鉴权风险方案"})
@RestController
@RequestMapping(value = "auditsvr/v2/riskProgramme")
public class RiskProgrammeController {

    public static Logger logger = LoggerFactory.getLogger(RiskProgrammeController.class);

    @Autowired
    private IRiskProgrammeFacade riskProgrammeFacade;

    @ApiOperation(value = "方案列表")
    @ApiImplicitParam(name = "searchDTO", value = "查询条件")
    @PostMapping(value = "_search")
    public Result<List<RiskProgrammeVo>> paging(SearchDTO searchDTO) {
        Page<RiskProgrammeVo> page = riskProgrammeFacade.paging(searchDTO);
        return Result.success(page.getTotalElements(), page.getContent());
    }

    @ApiOperation(value = "方案详情")
    @ApiImplicitParam(name = "id", value = "方案id")
    @GetMapping(value = "/{id}")
    public Result<RiskProgrammeVo> findOne(@PathVariable(value = "id") Long id) {
        return Result.success(riskProgrammeFacade.findOne(id));
    }

    @ApiOperation(value = "保存方案")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "方案id"),
            @ApiImplicitParam(name = "riskProgrammeDto", value = "方案内容"),
    })
    @PutMapping(value = "/{id}")
    public Result save(@PathVariable(value = "id") Long id, @Valid @RequestBody RiskProgrammeDto riskProgrammeDto) {
        riskProgrammeFacade.save(id, riskProgrammeDto);
        return Result.success("成功");
    }
}
