package com.dragoninfo.dcuc.authweb.restcontroller.audit.v2;

import com.dragoninfo.dcuc.auth.audit.dto.WarningProgrammeDto;
import com.dragoninfo.dcuc.auth.audit.facade.IWarningProgrammeFacade;
import com.dragoninfo.dcuc.auth.audit.vo.WarningProgrammeVo;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * @Author: qiuyu
 * @Date: 2021/5/25 14:31
 * @Description:
 */
@Api(tags = {"鉴权预警方案"})
@RestController
@RequestMapping(value = "auditsvr/v2/warningProgramme")
public class WarningProgrammeController {

    public static Logger logger = LoggerFactory.getLogger(WarningProgrammeController.class);

    @Autowired
    private IWarningProgrammeFacade warningProgrammeFacade;

    @ApiOperation(value = "方案列表")
    @ApiImplicitParam(name = "searchDTO", value = "查询条件")
    @PostMapping(value = "_search")
    public Result<List<WarningProgrammeVo>> paging(SearchDTO searchDTO) {
        Page<WarningProgrammeVo> page = warningProgrammeFacade.paging(searchDTO);
        return Result.success(page.getTotalElements(), page.getContent());
    }

    @ApiOperation(value = "方案详情")
    @ApiImplicitParam(name = "id", value = "方案id")
    @GetMapping(value = "/{id}")
    public Result<WarningProgrammeVo> findOne(@PathVariable(value = "id") Long id) {
        return Result.success(warningProgrammeFacade.findOne(id));
    }

    @ApiOperation(value = "保存方案")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "方案id"),
            @ApiImplicitParam(name = "warningProgrammeDto", value = "方案内容"),
    })
    @PutMapping(value = "/{id}")
    public Result save(@PathVariable Long id, @Valid @RequestBody WarningProgrammeDto warningProgrammeDto) {
        warningProgrammeFacade.save(id, warningProgrammeDto);
        return Result.success("成功");
    }
}
