package com.dragoninfo.dcuc.authweb.restcontroller.auth;


import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.auth.auth.enumresources.YesNotEnum;
import com.dragoninfo.dcuc.auth.auth.vo.ApplicationInfoVO;
import com.dragoninfo.dcuc.auth.power.facade.IApplicationInfoFacade;
import com.dragoninfo.dcuc.auth.sub.dto.AuthOrgDTO;
import com.dragoninfo.dcuc.auth.sub.dto.AuthUserDTO;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthOrgInfoFacade;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthUserInfoFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.auth.vo.ApplicationInfoVo;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.core.context.ContextUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Api(tags = {"授权模块-应用管理接口"})
@RestController
@RequestMapping(value = "authsvr/v2/applicationinfo")
public class ApplicationController {

    public static Logger logger = LoggerFactory.getLogger(ApplicationController.class);

    @Autowired
    private IApplicationInfoFacade iApplicationInfoFacade;
    @Autowired
    private IApplyInfoFacade applyInfoFacade;
    @Autowired
    private IAuthUserInfoFacade authUserInfoFacade;
    @Autowired
    private IAuthOrgInfoFacade authOrgInfoFacade;

    /**
     * 根据机构Id获取应用信息列表
     *
     * @param
     * @return
     */
    @ApiOperation(value = "根据机构code获取应用信息列表")
    @ApiImplicitParam(name = "searchable", value = "查询条件,orgCode")
    @PostMapping(value = "/appList/_search")
    public Result<List<ApplicationInfoVo>> getAppList(SearchDTO searchDTO) {
        List<ApplicationInfoVo> resultList = new ArrayList<>();
        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.addSearchFilter("deleted", SearchOperator.eq, YesNotEnum.NO.getValue());
        Page<ApplyInfo> applyInfos = applyInfoFacade.applyInfoPage(searchable.toSearchDTO());
        if (applyInfos.getContent().size() < 1) {
            return Result.success(new ArrayList<>());
        }
        //填充统计数据
        //TODO
        //DTO待设计
        List<ApplicationInfoVO> apps = iApplicationInfoFacade.addCountInfo(objectConvert(applyInfos));
        for (ApplicationInfoVO source : apps) {
            ApplicationInfoVo vo = new ApplicationInfoVo();
            BeanUtils.copyProperties(source, vo);
            resultList.add(vo);
        }
        return Result.success(applyInfos.getTotalElements(), resultList);
    }

    private List<ApplicationInfoVO> objectConvert(Page<ApplyInfo> applyInfos) {
        //TODO
        //需要填充权限中心的机构id
        //后续业务优化需要改掉
        List<String> orgCodes = applyInfos.getContent()
                .stream()
                .map(ApplyInfo::getOrgCode)
                .distinct()
                .collect(Collectors.toList());
        List<AuthOrgDTO> orgIdInfos = authOrgInfoFacade.convertCodesToIds(orgCodes);
        Map<String, String> codeIdMap = orgIdInfos
                .stream()
                .collect(Collectors.toMap(AuthOrgDTO::getCode, AuthOrgDTO::getId));
        List<ApplicationInfoVO> apps = new ArrayList<ApplicationInfoVO>();
        for (ApplyInfo applyInfo : applyInfos) {
            ApplicationInfoVO applicationInfoVO = new ApplicationInfoVO();
            applicationInfoVO.setId(applyInfo.getId());
            applicationInfoVO.setCode(applyInfo.getApplyCode());
            applicationInfoVO.setAppOrgId(codeIdMap.get(applyInfo.getOrgCode()));
            applicationInfoVO.setName(applyInfo.getApplyName());
            applicationInfoVO.setMenus(0L);
            applicationInfoVO.setRoles(0L);
            applicationInfoVO.setInitquotas(0L);
            applicationInfoVO.setQuotaed(0L);
            applicationInfoVO.setLeavequotas(0L);
            apps.add(applicationInfoVO);
        }
        return apps;
    }

    /**
     * 获取机构应用配额分配情况
     *
     * @param
     * @return
     */
    @ApiOperation(value = "根据机构Id获取应用信息列表")
    @ApiImplicitParam(name = "searchable", value = "查询条件,orgCode机构")
    @PostMapping(value = "/appQutoList/_search")
    public Result<List<ApplicationInfoVo>> getAppQutoList(SearchDTO searchDTO) {
        List<ApplicationInfoVo> resultList = new ArrayList<>();
        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.addSearchFilter("deleted", SearchOperator.eq, YesNotEnum.NO.getValue());
        Page<ApplyInfo> applyInfos = applyInfoFacade.applyInfoPage(searchable.toSearchDTO());
        if (applyInfos.getContent().size() < 1) {
            return Result.success(new ArrayList<>());
        }
        List<ApplicationInfoVO> apps = iApplicationInfoFacade.addCountInfoInLimitRole(objectConvert(applyInfos));
        for (ApplicationInfoVO source : apps) {
            ApplicationInfoVo vo = new ApplicationInfoVo();
            BeanUtils.copyProperties(source, vo);
            resultList.add(vo);
        }
        return Result.success(applyInfos.getTotalElements(), resultList);
    }

    /**
     * 获取所有机构应用
     *
     * @return
     */
    @ApiOperation(value = "获取所有机构应用")
    @ApiImplicitParam(name = "searchable", value = "查询条件")
    @PostMapping(value = "/appQutoList/all")
    public Result<List<ApplicationInfoVo>> getAllAppQutoList(SearchDTO searchDTO) {

        Searchable.toSearchable(searchDTO);
        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.addSearchFilter("deleted", SearchOperator.eq, YesNotEnum.NO.getValue());
        searchable.addSearchFilter("applyStatus", SearchOperator.eq, YesNotEnum.NO.getValue());
        //TODO
        //DTO待设计
        Result result = getAppQutoListByBusiness(searchable);

        return result;
    }

    public Result getAppQutoListByBusiness(Searchable searchable) {
        Page<ApplyInfo> retainList = getApplyInfos(searchable);
        //填充统计数据
        List<ApplicationInfoVO> apps = iApplicationInfoFacade.addCountInfo(objectConvert(retainList));
        List<ApplicationInfoVo> resultList = new ArrayList<>();
        for (ApplicationInfoVO source : apps) {
            ApplicationInfoVo vo = new ApplicationInfoVo();
            BeanUtils.copyProperties(source, vo);
            resultList.add(vo);
        }
        return Result.success( retainList.getTotalElements(), resultList);
    }

    /**
     * 根据登录人员警种，业务域获取应用
     * 应用保留  应用下拥有非治安警种角色的应用，应用下拥有角色业务域为空的应用，应用下的角色业务域和当前登录用户的业务域有交集的应用
     *
     * @param
     * @return
     */
    private Page<ApplyInfo> getApplyInfos(Searchable searchable) {
        Page<ApplyInfo> retainList = applyInfoFacade.findApplyList(searchable.toSearchDTO());
        return retainList;
    }


}
