package com.dragoninfo.dcuc.authweb.restcontroller.auth;

import com.alibaba.fastjson.JSON;
import com.dragoninfo.dcuc.auth.auth.dto.OrgQuotaOpeDTO;
import com.dragoninfo.dcuc.auth.auth.entity.OrgQuotaAuthInfo;
import com.dragoninfo.dcuc.auth.auth.facade.IOrgQuotaAuthFacade;
import com.dragoninfo.dcuc.auth.sub.dto.AuthOrgDTO;
import com.dragoninfo.dcuc.auth.sub.dto.AuthUserDTO;
import com.dragoninfo.dcuc.auth.sub.dto.OrgQuotaNumDTO;
import com.dragoninfo.dcuc.auth.sub.dto.OrgTreeNodeDTO;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthOrgInfoFacade;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthUserInfoFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.auth.vo.OrgQuotaAuthInfoVo;
import com.dragoninfo.dcuc.authweb.restcontroller.auth.vo.OrgQuotaOperateVo;
import com.dragoninfo.dcuc.authweb.restcontroller.sub.vo.org.OrgTreeNodeVo;
import com.dragoninfo.dcuc.duceap.facade.ICodeListResourceFacade;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.metadata.CodeRecord;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.exception.ApplicationException;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.core.context.ContextUtils;
import com.dragonsoft.duceap.web.annotation.Permission;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Api(tags = {"授权模块-机构配额管理"})
@RestController
@RequestMapping(value = "authsvr/v2/orgQuota/")
public class OrgQuotaAuthController {
    private final Logger logger=LoggerFactory.getLogger(this.getClass());

    @Autowired
    private IOrgQuotaAuthFacade iOrgQuotaAuthFacade;

    @Autowired
    private IAuthOrgInfoFacade authOrgInfoFacade;

    @Autowired
    private ICodeListResourceFacade iCodeListResourceFacade;

        /**
     * 机构配置-配额下发-计算
     * @param roleLevel 角色层级
     * @param unitClass 单位类别
     * @param orgId   应用所属机构id
     * @return
     */
    @GetMapping(value = "/searchOrg")
    @ApiImplicitParams({@ApiImplicitParam(name = "roleLevel", value = "角色层级", required = true),
            @ApiImplicitParam(name = "unitClass", value = "单位类别", required = true),
            @ApiImplicitParam(name = "orgId", value = "机构id"),
    })
    public Result<OrgQuotaNumDTO> searchOrg(
            @RequestParam(value = "orgId", required = false) String orgId,
            @RequestParam(value = "unitClass") String unitClass,
            @RequestParam(value = "roleLevel") String roleLevel) {
        SecurityUser user = UserContextUtils.getCurrentUser();
        try {
            OrgQuotaNumDTO orgQuotaNumsVo = authOrgInfoFacade.searchOrgForQuota(user.getId(), orgId, unitClass, roleLevel);
            return Result.success(orgQuotaNumsVo);
        } catch (Exception e) {
            logger.error("searchOrg异常", e);
            return Result.fail("计算失败");
        }
    }

    /**
     * 授权管理-机构配额-配额下发-机构树
     * @param unitClass
     * @param roleLevel
     * @param id        节点机构id
     * @return
     */
    @GetMapping(value = "calQuotaTreeYB")
    @ApiImplicitParams({@ApiImplicitParam(name = "roleLevel", value = "角色层级",required = false),
            @ApiImplicitParam(name = "unitClass", value = "单位类别",required = false),
            @ApiImplicitParam(name = "id", value = "机构id",required = false)
    })
    public Result<List<OrgTreeNodeVo>> calQuotaTreeYB(
            @RequestParam(value = "unitClass", required = false) String unitClass,
            @RequestParam(value = "roleLevel", required = false) String roleLevel,
            @RequestParam(value = "id", required = false) String id) {
        if (StringUtils.isEmpty(id)) {
            return Result.success(new ArrayList<>());
        } else {
            SecurityUser user = UserContextUtils.getCurrentUser();
            List<OrgTreeNodeDTO> calQuotaTree = authOrgInfoFacade.getCalQuotaTree(user.getId(), id, unitClass, roleLevel);
            List<OrgTreeNodeVo> collect = calQuotaTree.stream().map(e -> {
                OrgTreeNodeVo nodeVo = new OrgTreeNodeVo();
                BeanUtils.copyProperties(e, nodeVo);
                return nodeVo;
            }).collect(Collectors.toList());
            return Result.success(collect);
        }
    }


    /**
     * 获取指定机构配额分配信息
     *
     * @return
     */
    @ApiOperation(value = "获取指定机构配额分配信息")
    @PostMapping(value = "/orgQuotaInfos/_search")
    public Result<List<OrgQuotaAuthInfoVo>> getOrgQuotaInfos(@RequestBody OrgQuotaOperateVo operateVo) {
        String appId = operateVo.getAppId();
        String roleId = operateVo.getRoleId();
        String authOrgs = operateVo.getTargetOrgId();
        List<OrgQuotaAuthInfo> orgQuotaInfos = iOrgQuotaAuthFacade.getOrgQuotaInfos(appId, roleId, authOrgs);
        List<OrgQuotaAuthInfoVo> resultList = new ArrayList<>();
        for (OrgQuotaAuthInfo orgQuotaInfo : orgQuotaInfos) {
            OrgQuotaAuthInfoVo vo = new OrgQuotaAuthInfoVo();
            BeanUtils.copyProperties(orgQuotaInfo,vo);
            resultList.add(vo);
        }
        return Result.success(resultList);
    }

    /**
     * 保存机构配额分配信息
     * @param list 配额集合
     * @return
     */
    @ApiOperation(value = "保存机构配额分配信息")
    @PostMapping(value = "/orgQuota",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Result orgQuotaSave(@RequestBody List<OrgQuotaOperateVo> list) {
        SecurityUser curUser = UserContextUtils.getCurrentUser();
        //TODO
        //DTO待设计
        OrgQuotaOpeDTO redto = new OrgQuotaOpeDTO();
        redto.setOrgQuotaList(JSON.toJSONString(list));
        redto.setSecurityUser(curUser);
        ResponseStatus responseStatus = iOrgQuotaAuthFacade.updateOrgQuotaSave(redto);
        if(ResponseStatus.SUCCESS_CODE.equals(responseStatus.getStatusCode())){
            return Result.success(responseStatus.getMessage());
        }else {
            return Result.fail("300",responseStatus.getMessage());
        }
    }

    /**
     * 子级权限收回
     * @return
     */
    @ApiOperation(value = "子级权限收回")
    @Permission(value = "org_quota")
    @PostMapping(value = "/authCancelChild")
    public Result authCancelChild(@RequestBody OrgQuotaOperateVo orgQuotaOperateVo) {
        String appId = orgQuotaOperateVo.getAppId();
        String roleId =  orgQuotaOperateVo.getRoleId();
        String targetOrgId = orgQuotaOperateVo.getTargetOrgId();
        ResponseStatus responseMessage = iOrgQuotaAuthFacade.authCancelChild(appId, roleId, targetOrgId);
        if(ResponseStatus.SUCCESS_CODE.equals(responseMessage.getStatusCode())){
            return Result.success(responseMessage.getMessage());
        }else {
            return Result.fail(responseMessage.getStatusCode(),responseMessage.getMessage());
        }
    }

    @ApiOperation(value = "配额批量计算下发")
    @PostMapping(value = "/orgQuotaCalSave",produces = MediaType.APPLICATION_JSON_UTF8_VALUE )
    public Result orgQuotaCalSave(@RequestBody OrgQuotaOperateVo orgQuotaOperateVo) {
        SecurityUser userInfo = UserContextUtils.getCurrentUser();
        OrgQuotaOpeDTO orgQuotaOpeDTO = new OrgQuotaOpeDTO();
        BeanUtils.copyProperties(orgQuotaOperateVo,orgQuotaOpeDTO);
        orgQuotaOpeDTO.setSecurityUser(userInfo);
        ResponseStatus responseStatus = iOrgQuotaAuthFacade.orgQutaInfoCalSave(orgQuotaOpeDTO);
        if(ResponseStatus.SUCCESS_CODE.equals(responseStatus.getStatusCode())){
            return Result.success(responseStatus.getMessage());
        }else {
            return Result.fail(responseStatus.getStatusCode(),responseStatus.getMessage());
        }
    }

    /**
     * 根据筛选的条件确定分配范围可选项
     * @param orgId
     * @param roleLevel
     * @param unitClass
     * @return
     */
    @ApiOperation(value = "根据筛选的条件确定分配范围可选项")
    @ApiImplicitParams({@ApiImplicitParam(name = "orgId",required = true),
            @ApiImplicitParam(name = "unitClass", value = "机构层级",required = true),
            @ApiImplicitParam(name = "roleLevel", value = "角色层级",required = true),
    })
    @GetMapping(value = "/distributionRange")
    public Result<List<CodeRecord>> distributionRange(@RequestParam("orgId") String orgId,
                                                         @RequestParam("roleLevel") String roleLevel,
                                                         @RequestParam("unitClass") String unitClass) {
        AuthOrgDTO orgInfo = authOrgInfoFacade.get(orgId);
        int orgLevel = Integer.valueOf(orgInfo.getOrgLevel());
        int roleLevelInt = Integer.valueOf(roleLevel);
        int unit = 9;
        if (StringUtils.isNotBlank(unitClass)) {
            String[] unitClasss = unitClass.split(",");
            Arrays.sort(unitClasss, (o1, o2) -> {
                if (Integer.valueOf(o1) >= Integer.valueOf(o2)) {
                    return -1;
                } else {
                    return 1;
                }
            });
            unit = Integer.valueOf(unitClasss[0].substring(0, 1));
        }
        int end = roleLevelInt > unit ? unit : roleLevelInt;
        List<CodeRecord> codeList = iCodeListResourceFacade.listCode("code", "T_MD_ORG_LEVEL");
        List<CodeRecord> resultList = new ArrayList<>();
        for (CodeRecord codeRecord : codeList) {
            int value = Integer.valueOf(codeRecord.getValue());
            if (value <= end && value > orgLevel) {
                resultList.add(codeRecord);
            }
        }
        return Result.success(resultList);
    }

}
