package com.dragoninfo.dcuc.authweb.restcontroller.auth;

import com.dragoninfo.dcuc.app.dto.ServiceResourceDTO;
import com.dragoninfo.dcuc.app.entity.ServiceResource;
import com.dragoninfo.dcuc.app.facade.IServiceResourceFacade;
import com.dragoninfo.dcuc.app.vo.ServiceResourceVo;
import com.dragoninfo.dcuc.authweb.restcontroller.app.vo.ServiceResourceBaseVo;
import com.dragoninfo.dcuc.authweb.util.VersionUtils;
import com.dragoninfo.dcuc.duceap.facade.IDuceapUploadFacade;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author yica
 * @Date 2021/3/11 13:33
 **/
@Api(tags = {"服务资源管理接口"})
@RestController
@RequestMapping(value = "/authsvr/" + VersionUtils.VERSION_UID + "/resource/")
public class ServiceResourceController {
    public static Logger logger = LoggerFactory.getLogger(ServiceResourceController.class);

    @Autowired
    private IServiceResourceFacade serviceResourceFacade;

    @ApiOperation(value = "服务资源列表")
    @ApiImplicitParam(name = "serviceName", value = "查询条件")
    @PostMapping(value = "search/drop")
    public Result droplist(@RequestBody ServiceResourceBaseVo vo) {
        SearchDTO searchDTO = new SearchDTO();
        searchDTO.setSize("100");
        searchDTO.setPage("0");
        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.addSearchFilter("serviceName", SearchOperator.like, vo.getServiceName());
        Page<ServiceResource> serviceResources = serviceResourceFacade.resourcePages(searchable.toSearchDTO());
        List<ServiceResourceBaseVo> list = new ArrayList<>();
        serviceResources.getContent().forEach(item -> {
            ServiceResourceBaseVo resourceVo = new ServiceResourceBaseVo();
            BeanUtils.copyProperties(item, resourceVo);
            list.add(resourceVo);
        });
        return Result.success(list);
    }


    @ApiOperation(value = "服务资源分页列表")
    @ApiImplicitParam(name = "serviceName", value = "查询条件")
    @PostMapping(value = "/search")
    public Result page(Searchable searchable) {
        Page<ServiceResourceVo> page = serviceResourceFacade.page(searchable.toSearchDTO());
        return Result.success(page.getTotalElements(), page.getContent());
    }

    @ApiOperation(value = "服务资源新增")
    @ApiImplicitParam(name = "服务资源新增", value = "服务资源新增")
    @PostMapping(value = "/save")
    public Result save(@Validated @RequestBody ServiceResourceDTO serviceResourceDTO) {
        ResponseStatus responseStatus = serviceResourceFacade.save(serviceResourceDTO);
        if (!responseStatus.getStatusCode().equals("200")){
            return Result.fail(responseStatus.getStatusCode(), responseStatus.getMessage());
        }
        return Result.success();
    }

    @ApiOperation(value = "服务资源详情")
    @ApiImplicitParam(name = "服务资源详情", value = "服务资源详情")
    @GetMapping(value = "/{id}")
    public Result detailById(@PathVariable(name = "id") String id) {
        ServiceResourceVo  serviceResourceVo = serviceResourceFacade.detailById(id);
        return Result.success(serviceResourceVo);
    }

    @ApiOperation(value = "服务资源修改")
    @ApiImplicitParam(name = "服务资源修改", value = "服务资源修改")
    @PutMapping(value = "update")
    public Result update(@RequestBody ServiceResourceVo serviceResourceVo) {
        serviceResourceFacade.updateServiceResource(serviceResourceVo);
        return Result.success();
    }

    @ApiOperation(value = "服务资源删除")
    @ApiImplicitParam(name = "服务资源删除", value = "服务资源删除")
    @DeleteMapping(value = "/{id}")
    public Result deleteById(@PathVariable(name = "id") String id) {
        ResponseStatus  responseStatus = serviceResourceFacade.deleteById(id);
        if (!responseStatus.getStatusCode().equals("200")){
            return Result.fail(responseStatus.getStatusCode(), responseStatus.getMessage());
        }
        return Result.success();
    }
}
