package com.dragoninfo.dcuc.authweb.restcontroller.element;

import com.dragoninfo.dcuc.auth.element.facade.IEnvElementFacade;
import com.dragoninfo.dcuc.auth.element.vo.ElementUserRelRespVo;
import com.dragoninfo.dcuc.auth.element.vo.ElementUserSaveVo;
import com.dragoninfo.dcuc.auth.element.vo.EnvElementSaveVo;
import com.dragoninfo.dcuc.auth.element.vo.RespEnvElementVo;
import com.dragoninfo.dcuc.auth.sub.vo.AuthUserVo;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.core.search.filter.Condition;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author mazq
 * @date 2022/8/25
 */
@RestController
@RequestMapping("/envElement")
public class EnvElementController {

    @Autowired
    private IEnvElementFacade envElementFacade;

    @GetMapping("getById/{id}")
    public Result<RespEnvElementVo> getById(@PathVariable("id") String id) {
        RespEnvElementVo vo = envElementFacade.getById(id);
        return Result.success(vo);
    }

    @PostMapping("search")
    public Result<List<RespEnvElementVo>> pageSearch(SearchDTO searchDTO) {
        Page<RespEnvElementVo> page = envElementFacade.pageSearch(searchDTO);
        return Result.success(page.getTotalElements(), page.getContent());
    }

    @PostMapping("save")
    public Result save(@RequestBody EnvElementSaveVo envElementVo) {
       ResponseStatus responseStatus = envElementFacade.save(envElementVo);
        return getResult(responseStatus);
    }

    @PostMapping("update")
    public Result update(@RequestBody EnvElementSaveVo envElementVo) {
        ResponseStatus responseStatus = envElementFacade.update(envElementVo);
        return getResult(responseStatus);
    }

    @DeleteMapping("deleteById/{id}")
    public Result deleteById(@PathVariable("id") String id) {
        ResponseStatus responseStatus = envElementFacade.deleteById(id);
        return getResult(responseStatus);
    }

    private Result getResult(ResponseStatus responseStatus) {
        Result result = new Result();
        result.setMsg(responseStatus.getMessage());
        result.setResult(responseStatus.getStatusCode());
        return result;
    }

    @ApiOperation(value = "关联用户分页查询")
    @PostMapping("userRelPage")
    public Result<List<ElementUserRelRespVo>> userRelPage(SearchDTO searchDTO) {
        Searchable searchable = Searchable.toSearchable(searchDTO);
        Condition elementIdCondition = searchable.getSearchFilter("elementId", SearchOperator.eq);
        if (null == elementIdCondition) {
            return Result.failMessage("请选择环境要素");
        }
        Page<ElementUserRelRespVo> page = envElementFacade.userRelPage(searchDTO);
        return Result.success(page.getTotalElements(),page.getContent());
    }

    @ApiOperation(value = "非关联用户分页查询")
    @PostMapping("notInUserRelPage")
    public Result<List<AuthUserVo>> notInUserRelPage(SearchDTO searchDTO) {
        Searchable searchable = Searchable.toSearchable(searchDTO);
        Condition elementIdCondition = searchable.getSearchFilter("elementId", SearchOperator.eq);
        if (null == elementIdCondition) {
            return Result.failMessage("请选择环境要素");
        }
        Page<AuthUserVo> page = envElementFacade.notInUserRelPage(searchDTO);
        return Result.success(page.getTotalElements(),page.getContent());
    }

    @ApiOperation(value = "关联关系添加")
    @PostMapping("userRelSave")
    public Result<Object> userRelSave(@Valid @RequestBody ElementUserSaveVo relSaveVo) {
        envElementFacade.userRelSave(relSaveVo);
        return Result.success();
    }

    @ApiOperation(value = "关联关系移除")
    @DeleteMapping("deleteUserRel/{id}")
    public Result<Object> deleteUserRel(@PathVariable("id") String id) {
        envElementFacade.deleteUserRel(id);
        return Result.success();
    }
}
