package com.dragoninfo.dcuc.authweb.restcontroller.list.controller;

import com.dragoninfo.dcuc.auth.list.dto.AppFunRedListAddDTO;
import com.dragoninfo.dcuc.auth.list.dto.FunRedListTreeDTO;
import com.dragoninfo.dcuc.auth.list.dto.RedListOperateDTO;
import com.dragoninfo.dcuc.auth.list.facade.IRedListFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.list.vo.AppFunRedListAddVo;
import com.dragoninfo.dcuc.authweb.restcontroller.list.vo.FunRedListTreeVo;
import com.dragoninfo.dcuc.authweb.restcontroller.list.vo.RedListOperateVo;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.commons.util.collections.CollectionUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author mazq
 * @date 2021/7/12
 */
@Api(tags = "权限红名单接口")
@RestController
@RequestMapping(value = "/authsvr/v2/redlist")
public class RedListController {

    @Autowired
    IRedListFacade redListFacade;

    @ApiOperation(value = "功能红名单列表查询")
    @ApiImplicitParam(name = "searchable", value = "查询条件 level：= ; appName：like ; appCode：like")
    @PostMapping(value = "funRedListPage")
    public Result<List<FunRedListTreeVo>> funRedListPage(Searchable searchable) {
        Page<FunRedListTreeDTO> treeDTOPage = redListFacade.funRedListPage(searchable.toSearchDTO());
        List<FunRedListTreeDTO> content = treeDTOPage.getContent();
        List<FunRedListTreeVo> collect = content.stream().map(item -> {
            FunRedListTreeVo vo = new FunRedListTreeVo();
            BeanUtils.copyProperties(item, vo);
            return vo;
        }).collect(Collectors.toList());
        return Result.success(treeDTOPage.getTotalElements(), collect);
    }

    @ApiOperation(value = "功能红名单添加")
    @PostMapping("funRedListAdd")
    public Result userWhiteListAdd(@RequestBody List<AppFunRedListAddVo> addVoList) {
        if(CollectionUtils.isEmpty(addVoList)) {
            return Result.fail();
        }
        List<AppFunRedListAddDTO> collect = addVoList.stream().map(item -> {
            AppFunRedListAddDTO addDTO = new AppFunRedListAddDTO();
            BeanUtils.copyProperties(item, addDTO);
            return addDTO;
        }).collect(Collectors.toList());
        redListFacade.funRedListAdd(collect);
        return Result.success();
    }

    @ApiOperation(value = "红名单批量修改等级")
    @PostMapping("redListUpdateLevel")
    public Result redListUpdateLevel(@RequestBody RedListOperateVo vo) {
        RedListOperateDTO dto = new RedListOperateDTO();
        BeanUtils.copyProperties(vo, dto);
        redListFacade.funRedListUpdateLevel(dto);
        return Result.success();
    }

    @ApiOperation(value = "应用功能红名单批量移除")
    @PostMapping("funRedListBatchDel")
    public Result funRedListBatchDel(@RequestBody RedListOperateVo vo) {
        RedListOperateDTO dto = new RedListOperateDTO();
        BeanUtils.copyProperties(vo, dto);
        redListFacade.funRedListBatchDel(dto);
        return Result.success();
    }

    @ApiOperation(value = "应用功能红名单单条记录移除")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "resourceType",value = "app menu 资源类型"),
            @ApiImplicitParam(name = "resourceId",value = "资源id 应用或是功能id"),
            @ApiImplicitParam(name = "isTreeNode",value = "是否是树节点")
    })
    @DeleteMapping(value = "funRedListDel")
    public Result funRedListDel(@RequestParam("resourceType") String resourceType,
                                @RequestParam("resourceId") String resourceId,
                                @RequestParam("isTreeNode") Boolean isTreeNode) {
        redListFacade.funRedListDel(resourceType, resourceId, isTreeNode);
        return Result.success();
    }

    @ApiOperation(value = "查询已被添加为红名单的功能id")
    @ApiImplicitParam(name = "appId", value = "appId 应用id 非必选,不填时查询所有")
    @GetMapping("funIdsInRedList")
    public Result<List<String>> funIdsInRedList(@RequestParam(value = "appId", required = false) String appId) {
        List<String> funIds = redListFacade.funIdsInRedList(appId);
        return Result.success(funIds);
    }


}
