package com.dragoninfo.dcuc.authweb.restcontroller.list.controller;

import com.dragoninfo.dcuc.auth.list.dto.WhiteListOperateDTO;
import com.dragoninfo.dcuc.auth.list.dto.WhiteListUserDTO;
import com.dragoninfo.dcuc.auth.list.facade.IWhiteListFacade;
import com.dragoninfo.dcuc.auth.sub.enumresource.SubObjTypeEnum;
import com.dragoninfo.dcuc.authweb.restcontroller.list.vo.WhiteListOperateVo;
import com.dragoninfo.dcuc.authweb.restcontroller.list.vo.WhiteListUserVo;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.core.search.Searchable;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author mazq
 * @date 2021/7/9
 */
@Api(tags = "权限白名单接口")
@RestController
@RequestMapping(value = "/authsvr/v2/whitelist")
public class WhiteListController {

    @Autowired
    IWhiteListFacade whiteListFacade;

    @ApiImplicitParam(name = "searchable", value = "查询条件 level：= ; name：like ; idcard：like ; orgId：=")
    @ApiOperation(value = "用户白名单列表查询")
    @PostMapping("userWhiteListPage")
    public Page<WhiteListUserVo> whiteListUserPage(Searchable searchable) {
        Page<WhiteListUserDTO> userPage = whiteListFacade.whiteListUserPage(searchable.toSearchDTO());
        List<WhiteListUserDTO> content = userPage.getContent();
        List<WhiteListUserVo> collect = content.stream().map(item -> {
            WhiteListUserVo userVo = new WhiteListUserVo();
            BeanUtils.copyProperties(item, userVo);
            return userVo;
        }).collect(Collectors.toList());
        return new PageImpl<>(collect, userPage.getPageable(), userPage.getTotalElements());
    }

    @ApiOperation(value = "用户白名单添加")
    @PostMapping("userWhiteListAdd")
    public Result userWhiteListAdd(@RequestBody WhiteListOperateVo vo) {
        WhiteListOperateDTO dto = new WhiteListOperateDTO();
        vo.setSubType(SubObjTypeEnum.SUB_USER.getCode());
        BeanUtils.copyProperties(vo, dto);
        whiteListFacade.whiteListAdd(dto);
        return Result.success();
    }

    @ApiOperation(value = "白名单批量删除")
    @PostMapping("whiteListDel")
    public Result whiteListDel(@RequestBody WhiteListOperateVo vo) {
        WhiteListOperateDTO dto = new WhiteListOperateDTO();
        BeanUtils.copyProperties(vo, dto);
        whiteListFacade.whiteListDel(dto);
        return Result.success();
    }

    @ApiOperation(value = "白名单批量修改等级")
    @PostMapping("whiteListUpdateLevel")
    public Result whiteListUpdateLevel(@RequestBody WhiteListOperateVo vo) {
        WhiteListOperateDTO dto = new WhiteListOperateDTO();
        BeanUtils.copyProperties(vo, dto);
        whiteListFacade.whiteListUpdateLevel(dto);
        return Result.success();
    }

}
