package com.dragoninfo.dcuc.authweb.restcontroller.login;


import com.dragoninfo.dcuc.auth.admin.facade.IManageInfoFacade;
import com.dragoninfo.dcuc.auth.auth.dto.AuthMenuDTO;
import com.dragoninfo.dcuc.auth.auth.facade.IAuthMenuFacade;
import com.dragoninfo.dcuc.auth.auth.facade.IRoleFacade;
import com.dragoninfo.dcuc.auth.sub.dto.AuthUserContactDTO;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthUserInfoFacade;
import com.dragoninfo.dcuc.authweb.config.DcucAuthWebConfig;
import com.dragoninfo.dcuc.authweb.util.HeadTokenUtils;
import com.dragoninfo.dcuc.common.enums.UserExtInfoEnum;
import com.dragonsoft.duceap.base.entity.security.SecurityRight;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.collections.CollectionUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping(value = "/user")
public class DcucLoginController {

    @Autowired
    private IAuthMenuFacade menuFacade;

    @Autowired
    private IAuthUserInfoFacade authUserInfoFacade;

    @Autowired
    private IRoleFacade iRoleFacade;

    @Autowired
    private DcucAuthWebConfig dcucAuthWebConfig;

    @Autowired
    private IManageInfoFacade manageInfoFacade;

    public static final Logger logger = LoggerFactory.getLogger(DcucLoginController.class);

    public DcucLoginController() {
    }

    @GetMapping(value = "/info")
    public SecurityUser info(HttpServletRequest request) {

        String userToken = HeadTokenUtils.getUserToken(request);
        String appToken = HeadTokenUtils.getAppToken(request);
        logger.info("用户令牌id:{} , 应用令牌id:{}", userToken, appToken);
        try {
            SecurityUser securityUser = UserContextUtils.getCurrentUser();
            String idcard = securityUser.getIdcard();
            AuthUserContactDTO userInfo = authUserInfoFacade.getUserWithContactByIdcard(idcard);
            logger.debug("securityUser:{}", JsonUtils.toJSONString(securityUser));
            //从dcuc获取用户数据,使用T_AUTH_USER_INFO表中的数据补充SecurityUser字段
            //将securityUser中的id改为权限中心用户的id
            securityUser.setId(userInfo.getId());
            securityUser.setManId(userInfo.getId());
            securityUser.setDeptId(userInfo.getOrgId());
            securityUser.setSecurityOrg(userInfo.getOrgCode());
            securityUser.setUserType(userInfo.getUserType());
            securityUser.setMobile(StringUtils.isBlank(userInfo.getMobileWork()) ? userInfo.getMobilePrivate() : userInfo.getMobileWork());
            List<SecurityRight> authmenu = getAuthmenu(securityUser);
            securityUser.setSecurityRightList(authmenu);

            Map<String, Object> map = new HashMap<>(UserExtInfoEnum.values().length);

            map.put(UserExtInfoEnum.USER_TOKEN_ID.getValue(), userToken);
            map.put(UserExtInfoEnum.APP_TOKEN_ID.getValue(), appToken);
            securityUser.setExtendtions(map);

            logger.info("securityUser:{}", JsonUtils.toJSONString(securityUser));
            UserContextUtils.setCurrentUser(securityUser);
            return securityUser;
        } catch (Exception var3) {
            logger.error("获取用户信息异常", var3);
            throw new SecurityException("获取用户信息异常", var3);
        }

    }


    //获取菜单
    private List<SecurityRight> getAuthmenu(SecurityUser securityUser) {
        List<AuthMenuDTO> authMenuInfos;
        //用户中心菜单
        //ADMIN
        if (isRootUser(securityUser.getId())) {
            authMenuInfos = menuFacade.getAllMenus();
        } else {
            logger.info("进入权限授权模块控制菜单权限");
            //使用权限授权模块控制菜单权限
            String userId = securityUser.getId();
            String appCode = dcucAuthWebConfig.getAppCode();
            logger.debug("Get MenuInfo userId:{} , appCode:{}", userId, appCode);
            List<String> appFunCodes = iRoleFacade.getAuthByUserIdAndAppCode(userId, appCode);
            authMenuInfos = menuFacade.getMenuByCods(appFunCodes);
        }

        if(CollectionUtils.isNotEmpty(authMenuInfos)) {
            List<String> collect = authMenuInfos.stream().map(AuthMenuDTO::getCode).collect(Collectors.toList());
            String[] codes = collect.toArray(new String[0]);
            securityUser.setSecurityRights(codes);
        }

        return Optional.ofNullable(authMenuInfos).orElse(new ArrayList<>())
                .stream().map(this::getSecurityRight).collect(Collectors.toList());
    }

    private SecurityRight getSecurityRight(AuthMenuDTO menuInfo) {
        SecurityRight securityRight = new SecurityRight();
        securityRight.setId(menuInfo.getId());
        securityRight.setCode(menuInfo.getCode());
        securityRight.setUrl(menuInfo.getUrl());
        securityRight.setApplicationId(menuInfo.getAppId());
        securityRight.setpId(menuInfo.getParentId());
        securityRight.setSortNum(menuInfo.getSeq());
        securityRight.setShowMode(menuInfo.getShowMode());
        securityRight.setIsSystem(menuInfo.getIsSystem());
        return securityRight;
    }

    private boolean isRootUser(String id) {
        return authUserInfoFacade.isRootUser(id);
    }

}
