package com.dragoninfo.dcuc.authweb.restcontroller.resource;

import com.dragoninfo.dcuc.coder.facace.IMdDataSourceFacade;
import com.dragoninfo.dcuc.duceap.facade.IDuceapMdDataSourceFacade;
import com.dragoninfo.dcuc.duceap.metedata.dto.MdDatasourceDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.metadata.MdDatasourceVO;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.commons.util.UUIDUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.core.entity.response.ResponseResult;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.web.controller.BaseController;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

/**
 * <pre>
 *  数据源controller类，
 * </pre>
 * @author huangzqa
 */
@Controller
@RequestMapping(value = "/duacm/datasource")
public class DataSourceController extends BaseController {

    @Autowired
    private IMdDataSourceFacade iMdDataSourceFacade;
    @Autowired
    private IDuceapMdDataSourceFacade iDuceapMdDataSourceFacade;
    /**
     * 重写父类方法，若不做其他处理可不用重写
     * @param searchable
     * @return
     */
    @RequestMapping(value = "list")
    @ResponseBody
    public ResponseResult list(Searchable searchable) {
        SearchDTO searchDTO = searchable.toSearchDTO();
        Page<MdDatasourceDTO> objects = iMdDataSourceFacade.paging(searchDTO);
        return ResponseResult.newInstance(ResponseResult.toConvert(objects, searchable));
    }

    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "id", value = "表码ID", required = true
            , example = "40288a8b699fc2500169a33b20540000")})
    @RequestMapping(value = "editRow/{id}")
    public String editRow(@PathVariable("id") String id, Model model) {
        MdDatasourceDTO mdDataSource = iDuceapMdDataSourceFacade.get(id);
        model.addAttribute("model", mdDataSource);
        return "resourceManagement/datasource/dataSourceEdit";
    }

    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "id", value = "表码ID", required = true
            , example = "40288a8b699fc2500169a33b20540000")})
    @RequestMapping(value = "viewRow/{id}")
    public String viewRow(@PathVariable("id") String id, Model model) {
        MdDatasourceDTO mdDataSource = iDuceapMdDataSourceFacade.get(id);
        StringBuilder password = new StringBuilder();
        for (int i = 0; i < mdDataSource.getPassword().length(); i++) {
            password.append("*");
        }
        mdDataSource.setPassword(password.toString());
        model.addAttribute("model", mdDataSource);
        return "resourceManagement/datasource/dataSourceView";
    }

    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "id", value = "表码ID", required = true
            , example = "40288a8b699fc2500169a33b20540000")})
    @RequestMapping(value = "delete/{id}")
    @ResponseBody
    public ResponseStatus delete(@PathVariable("id") String id) {
        if (StringUtils.isNotEmpty(id)) {
            try {
                iMdDataSourceFacade.delete(id);
            } catch (UnsupportedOperationException e) {
                return ResponseStatus.fail(e.getMessage());
            }
            return ResponseStatus.success("删除成功");
        }
        return ResponseStatus.fail("请选择要删除的数据源");
    }

    @RequestMapping(value = "updateAll")
    @ResponseBody
    public ResponseStatus updateAll(Model model, @Valid @ModelAttribute("m") MdDatasourceVO data, HttpServletRequest servletRequest) {
        //业务逻辑处理
        //若不需要进行其他业务逻辑处理 可不用重写该方法
        if (StringUtils.isEmpty(data.getIsSystem())) {
            data.setIsSystem("0");
        }
        if (StringUtils.isEmpty(data.getIsDeleted())) {
            data.setIsDeleted("0");
        }
        MdDatasourceDTO dto = new MdDatasourceDTO();
        BeanUtils.copyProperties(data,dto);
        iDuceapMdDataSourceFacade.update(dto);
        return ResponseStatus.success("更新成功");
    }

    @RequestMapping(value = "createAll")
    @ResponseBody
    public ResponseStatus createAll(Model model, @Valid MdDatasourceVO data) {
        data.setDatasourceId(UUIDUtils.getUUID());
        data.setIsSystem("0");
        data.setIsDeleted("0");
        MdDatasourceDTO dto = new MdDatasourceDTO();
        BeanUtils.copyProperties(data,dto);
        iDuceapMdDataSourceFacade.save(dto);
        return ResponseStatus.success("保存成功");
    }

    @RequestMapping(value = "export")
    public void listExport(SearchDTO searchDTO, Model modelde, HttpServletRequest request, HttpServletResponse response) throws IOException {
        //若需要对导出数据进行处理，可在此编写代码 ；
        String fileName = request.getParameter("fileName");
        Searchable searchable = Searchable.toSearchable(searchDTO);
        List<MdDatasourceDTO> result = iMdDataSourceFacade.findList(searchable);
        //调用父类导出Excel，同时根据元数据信息进行表码翻译
        this.exportExcel(fileName, result, searchable.getMetaData(), request, response);
    }

    @RequestMapping(value = "testConn")
    @ResponseBody
    public ResponseStatus testConn(@Valid @ModelAttribute("m") MdDatasourceVO data, HttpServletRequest servletRequest) {
        MdDatasourceDTO dto = new MdDatasourceDTO();
        BeanUtils.copyProperties(data,dto);
        ResponseStatus responseStatus = iMdDataSourceFacade.testConnetion(dto);
        return responseStatus;
    }
}