package com.dragoninfo.dcuc.authweb.restcontroller.resource;

import cn.hutool.core.bean.BeanUtil;
import com.dragoninfo.dcuc.authweb.restcontroller.resource.vo.CodeDicVo;
import com.dragoninfo.dcuc.coder.facace.ICodeInfoFacade;
import com.dragoninfo.dcuc.duceap.facade.IDuceapCodeFacade;
import com.dragoninfo.duceap.core.enums.ResultEnum;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.metadata.CodeDicDTO;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 表码
 */
@Api(tags = {"表码表管理接口"})
@RestController
@RequestMapping(value = "/codesvr")
public class MdDynCodeDicController extends BaseController {

    @Autowired
    private ICodeInfoFacade iCodeInfoFacade;
    @Autowired
    private IDuceapCodeFacade iDuceapCodeFacade;

    /**
     * 表码列表
     *
     * @param searchDTO
     * @return
     */
    @ApiOperation(value = "查询表码列表")
    @ApiImplicitParam(name = "searchable", value = "查询条件")
    @PostMapping(value = "/codes")
    public Result<List<CodeDicVo>> pageDynCodeVO(SearchDTO searchDTO) {
        Page<CodeDicDTO> orgCodePage = iCodeInfoFacade.page(searchDTO);

        List<CodeDicDTO> content = orgCodePage.getContent();

        List<CodeDicVo> codeDicVoList = new ArrayList<>(content.size());
        for (CodeDicDTO codeDicDTO : content) {
            CodeDicVo codeDicVo = new CodeDicVo();
            BeanUtil.copyProperties(codeDicDTO, codeDicVo);
            codeDicVoList.add(codeDicVo);
        }

        return Result.success(orgCodePage.getTotalElements(), codeDicVoList);
    }

    @ApiOperation(value = "修改表码")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "id", value = "表码ID", required = true
            , example = "40288a8b699fc2500169a33b20540000")})
    @PutMapping(value = "/code/{id}")
    public Result update(CodeDicVo codeDicVo) {
        try {
            if (codeDicVo == null) {
                return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "空数据，修改失败");
            }
            CodeDicDTO codeDic = new CodeDicDTO();
            BeanUtil.copyProperties(codeDicVo, codeDic);
            iDuceapCodeFacade.save(codeDic);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "修改异常");
        }
        return Result.success();
    }

    @ApiOperation(value = "保存表码")
    @ApiImplicitParam(name = "codeDicVo", value = "表码vo")
    @PostMapping(value = "/code")
    public Result save(CodeDicVo codeDicVo) {
        try {
            if (codeDicVo == null) {
                return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "空数据，保存失败");
            }
            CodeDicDTO codeDic = new CodeDicDTO();
            BeanUtil.copyProperties(codeDicVo, codeDic);
            iDuceapCodeFacade.update(codeDic);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "保存异常");
        }
        return Result.success();
    }

    @ApiOperation(value = "批量删除表码")
    @ApiImplicitParam(name = "批量id", value = "ids")
    @PostMapping(value = "/code/clean")
    public Result deleteRow(@RequestBody String[] ids) {
        if (ids == null) {
            return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "删除失败，此次删除的记录数为0");
        } else {
            try {
                for (String id : ids) {
                    iCodeInfoFacade.delete(id);
                }
            } catch (Exception e) {
                logger.error(e.getMessage(), e);
                return Result.fail(ResultEnum.SERVICE_FAIL.getValue(), "修改异常");
            }

        }
        return Result.success();
    }

    /**
     * 表码信息页面
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "查询表码")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "id", value = "表码ID", required = true
            , example = "40288a8b699fc2500169a33b20540000")})
    @GetMapping(value = "/code/{id}")
    public Result<CodeDicVo> getModel(@PathVariable("id") String id) {
        CodeDicDTO codeDic = iDuceapCodeFacade.get(id);
        CodeDicVo codeDicVo = new CodeDicVo();
        BeanUtil.copyProperties(codeDic, codeDicVo);
        return Result.success(codeDicVo);
    }

}