package com.dragoninfo.dcuc.authweb.restcontroller.sub;

import cn.hutool.core.util.StrUtil;
import com.dragoninfo.dcuc.auth.sub.dto.AuthOrgDTO;
import com.dragoninfo.dcuc.auth.sub.dto.OrgTreeNodeDTO;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthOrgInfoFacade;
import com.dragoninfo.dcuc.authweb.business.impl.MtAuthBusiness;
import com.dragoninfo.dcuc.authweb.common.SysConstants;
import com.dragoninfo.dcuc.authweb.restcontroller.org.vo.OrgRangeVo;
import com.dragoninfo.dcuc.authweb.restcontroller.sub.vo.org.AuthOrgVo;
import com.dragoninfo.dcuc.authweb.restcontroller.sub.vo.org.OrgTreeNodeVo;
import com.dragoninfo.dcuc.authweb.restcontroller.sub.vo.org.TreeReqVo;
import com.dragoninfo.dcuc.authweb.util.VersionUtils;
import com.dragoninfo.duceap.commons.util.server.OrgInfoUtil;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author wangrs
 * @date 2021-04-27
 */
@Api(tags = {"授权管理-主客体管理-机构信息"})
@RestController
@RequestMapping(value = "/authsvr/" + VersionUtils.VERSION_UID + "/authorginfo")
public class AuthOrgInfoController {


    @Autowired
    private IAuthOrgInfoFacade authOrgInfoFacade;

    @Autowired
    private MtAuthBusiness mtAuthBusiness;

    @PostMapping("/page")
    @ApiOperation(value = "服务主体-机构信息列表")
    @ApiImplicitParam(name = "searchDTO", value = "查询条件")
    public Page<AuthOrgVo> page(SearchDTO searchDTO) {
        Page<AuthOrgDTO> page = authOrgInfoFacade.page(searchDTO);
        List<AuthOrgDTO> content = page.getContent();
        List<AuthOrgVo> collect = content.stream().map(e -> {
            AuthOrgVo authOrgVo = new AuthOrgVo();
            BeanUtils.copyProperties(e, authOrgVo);
            return authOrgVo;
        }).collect(Collectors.toList());
        return new PageImpl<>(collect, page.getPageable(),page.getTotalElements());
    }

    @GetMapping(value = "/detail/{id}")
    @ApiImplicitParam(name = "id", value = "机构信息ID")
    @ApiOperation(value = "服务主体-机构信息详情")
    public AuthOrgVo detail(@PathVariable(value = "id") String id) {
        AuthOrgDTO authOrgDTO = authOrgInfoFacade.get(id);
        AuthOrgVo authOrgVo = new AuthOrgVo();
        BeanUtils.copyProperties(authOrgDTO, authOrgVo);
        return authOrgVo;
    }

    @DeleteMapping(value = "/delete/{id}")
    @ApiImplicitParam(name = "id", value = "机构信息ID")
    @ApiOperation(value = "服务主体-机构信息删除")
    public ResponseStatus delete(@PathVariable(value = "id") String id) {
        authOrgInfoFacade.deleteById(id);
        return ResponseStatus.success();
    }

    /**
     * 获取单个树节点机构信息
     */
    @ApiOperation(value = "单个树节点信息")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "id", value = "机构ID", required = true
            , example = "40288a8b699fc2500169a33b20540000")})
    @GetMapping(value = "org/tree/node/{id}")
    public Result<OrgTreeNodeVo> orgTreeNode(@PathVariable("id") String id) {
        OrgTreeNodeDTO orgTreeNode = authOrgInfoFacade.getOrgTreeNode(id);
        OrgTreeNodeVo vo = new OrgTreeNodeVo();
        BeanUtils.copyProperties(orgTreeNode, vo);
        return Result.success(vo);
    }

    /**
     * 当前用户的管理范围机构树
     *
     * @param params type:机构树类型 (APP:权限管理范围 MGE:管理员管理范围 ), id:异步加载的树节点
     * @return
     */
    @ApiOperation(value = "机构树")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id"),
            @ApiImplicitParam(name = "type", value = "type:机构树类型(APP:权限管理范围)")})
    @PostMapping(value = "org/tree/catalog")
    public Result orgMtTreeYB(@RequestBody Map params) {
        String id = (String) params.get("id");
        String type = (String) params.get("type");
        //获取管理范围ids
        SecurityUser curUser = UserContextUtils.getCurrentUser();
        String mtAuthIds = mtAuthBusiness.getMtAuth(curUser.getId(), type);
        if (StringUtils.isEmpty(id)) {
            //获取机构树根节点列表
            List<OrgTreeNodeVo> result = this.treeRootList(mtAuthIds);
            return Result.success(result);
        } else {
            List<OrgTreeNodeVo> result = this.getChildTreeNode(id, mtAuthIds);
            return Result.success(result);
        }
    }

    /**
     * 管理员管理—管理范围—分配树
     *
     * @return
     */
    @ApiOperation(value = "管理员管理—管理范围—分配树")
    @PostMapping(value = "org/tree/admin")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id"),
            @ApiImplicitParam(name = "userId", value = "用户ID", required = true),
            @ApiImplicitParam(name = "mtType", value = "type:机构树类型(APP:权限管理范围)", required = true)
    })
    public Result orgMtTreeWithChecked(@RequestBody OrgRangeVo orgRangeVo) {
        String id = orgRangeVo.getId();
        String mtType = orgRangeVo.getMtType();
        String userId = orgRangeVo.getUserId();
        //获取管理范围ids
        SecurityUser curUser = UserContextUtils.getCurrentUser();
        String mtAuthIds = mtAuthBusiness.getMtAuth(curUser.getId(), mtType);
        //当临时表没有数据时候，获取登录人的权限范围
        if (StringUtils.isEmpty(mtAuthIds) && SysConstants.MT_TEMP.equals(orgRangeVo.getMtType())) {
            mtAuthIds = mtAuthBusiness.getMtAuth(curUser.getId(), SysConstants.MT_APP);
        }
        if (StringUtils.isEmpty(id)) {
            //获取机构树根节点列表
            List<OrgTreeNodeVo> result = this.treeRootList(mtAuthIds);
            return Result.success(checkTreeNode(result, mtType, userId));
        } else {
            List<OrgTreeNodeVo> result  = this.getChildTreeNode(id, mtAuthIds);
            return Result.success(checkTreeNode(result, mtType, userId));
        }
    }

    @ApiOperation(value = "机构树匹配")
    @ApiImplicitParams({@ApiImplicitParam(name = "name", value = "机构id"),
            @ApiImplicitParam(name = "mtType", value = "机构树类型 app:权限管理范围"),
            @ApiImplicitParam(name = "pageNum", value = "条数"),
            @ApiImplicitParam(name = "pageSize", value = "页数")
    })
    @PostMapping(value = "org/tree/match")
    public Result orgMatch(@RequestBody TreeReqVo reqVo) {
        String mtType = reqVo.getMtType();
        String name = reqVo.getName();
        Integer pageNum = reqVo.getPageNum();
        Integer pageSize = reqVo.getPageSize();
        String mts = "";
        String id = UserContextUtils.getCurrentUser().getId();
        if (StringUtils.isBlank(mtType)) {
            mts = "";
        } else {
            mts = mtAuthBusiness.getMtAuth(id, SysConstants.MT_APP);
        }
        if (StringUtils.isBlank(mts) && !StringUtils.isBlank(mtType)) {
            return Result.success();
        }
        ResponseDTO responseDTO = authOrgInfoFacade.orgMatch(name, mts, pageNum, pageSize);
        return Result.success(StringUtils.getLong(responseDTO.getMessage()), responseDTO.getResult());
    }

    /**
     * 管理员管理-管理范围-节点状态勾选
     *
     * @param treeNodes
     * @param type         机构树类型
     * @param targetUserId
     * @return
     */
    public List<OrgTreeNodeVo> checkTreeNode(List<OrgTreeNodeVo> treeNodes, String type, String targetUserId) {
        String mtAuthIds = mtAuthBusiness.getMtAuth(targetUserId, type);
        for (OrgTreeNodeVo treeNode : treeNodes) {
            String orgId = treeNode.getId();
            String path =  treeNode.getPath();
            //半勾选
            if (StringUtils.isNotEmpty(mtAuthIds) && mtAuthIds.contains(orgId + ":1")) {
                treeNode.setChecked("1");
            } else if (OrgInfoUtil.isHaveAuth(orgId, path, mtAuthIds)) {
                //全勾选
                treeNode.setChecked("2");
            } else {
                //未勾选
                treeNode.setChecked("0");
            }
        }
        return treeNodes;
    }


    /**
     * 根据ids获取初始树根节点
     *
     * @param mtAuthIds ：管理范围ids
     * @return
     */
    public List<OrgTreeNodeVo> treeRootList(String mtAuthIds) {
        if (StringUtils.isBlank(mtAuthIds)) {
            return new ArrayList<>();
        }
        List<String> orgIds = new ArrayList<>();
        for (String orgIdStr : mtAuthIds.split(StrUtil.COMMA)) {
            String[] orgIdStatus = orgIdStr.split(StrUtil.COLON);
            orgIds.add(orgIdStatus[0]);
        }
        List<OrgTreeNodeDTO> orgInfosList = authOrgInfoFacade.getSelfAndChildByIds(orgIds);
        //组成树结构只返回顶部节点
        //重新设置权限过滤后的子节点集合和isParent字段
        orgInfosList = convertFilterTree(orgInfosList, mtAuthIds, true);
        List<OrgTreeNodeVo> collect = orgInfosList.stream().map(e -> {
            e.setChildIds(null);
            OrgTreeNodeVo vo = new OrgTreeNodeVo();
            BeanUtils.copyProperties(e, vo);
            return vo;
        }).collect(Collectors.toList());
        return collect;
    }

    /**
     * 异步获取子节点
     *
     * @param id  父节点id
     * @param mtAuthIds 管理范围，为""或null时不过滤
     * @return
     */
    public List<OrgTreeNodeVo> getChildTreeNode(String id, String mtAuthIds) {
        //子级机构
        List<OrgTreeNodeDTO> childList = authOrgInfoFacade.getChildById(id, 2);
        childList = convertFilterTree(childList, mtAuthIds, true);
        return childList.stream().map(e->{
            OrgTreeNodeVo nodeVo = new OrgTreeNodeVo();
            BeanUtils.copyProperties(e, nodeVo);
            return nodeVo;
        }).collect(Collectors.toList());
    }

    /**
     * 组成树结构后只返回顶部节点集合
     * 组成树结构后重新设置权限过滤后的顶节点isParent字段和child字段
     * @param treeNodeList
     * @param mtAuthIds
     * @param needTree 是否需要组装成树结构
     * @return
     */
    private List<OrgTreeNodeDTO> convertFilterTree(List<OrgTreeNodeDTO> treeNodeList, String mtAuthIds, boolean needTree) {
        List<OrgTreeNodeDTO> collect = new ArrayList<>();
        if(StringUtils.isEmpty(mtAuthIds)) {
            return collect;
        }
        //筛选有权限的节点
        collect = treeNodeList
                .stream()
                .filter(e -> OrgInfoUtil.isHaveAuth(e.getId(), e.getPath(), mtAuthIds))
                .collect(Collectors.toList());
        if(!needTree) {
            return collect;
        } else {
            //重新设置权限过滤后的顶节点isParent字段和child字段
            //只返回顶部节点集合
            Map<String, OrgTreeNodeDTO> idMap = collect
                    .stream()
                    .collect(Collectors.toMap(OrgTreeNodeDTO::getId, e -> e, (old, last)->last));
            Set<String> keySet = idMap.keySet();
            //筛选父节点
            List<OrgTreeNodeDTO> allTopNodes = collect.stream().filter(e->{
                String pid = e.getPid();
                if(StringUtils.isBlank(pid)) {
                    return true;
                }
                if(!keySet.contains(pid)) {
                    return true;
                }
                return false;
            }).collect(Collectors.toList());
            //重新设置父节点isParent字段
            allTopNodes.forEach(e->{
                List<String> childIds = e.getChildIds();
                e.setIsParent(childIds.stream().anyMatch(keySet::contains));
                e.setChildIds(null);
            });
            return allTopNodes;
        }
    }
}
