package com.dragoninfo.dcuc.authweb.restcontroller.sub;

import com.dragoninfo.dcuc.auth.sub.facade.IAuthSubTaskTypeFacade;
import com.dragoninfo.dcuc.auth.sub.vo.tasktype.AuthSubTaskTypeVo;
import com.dragoninfo.dcuc.auth.sub.vo.tasktype.TaskTypeTreeVo;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author mazq
 * @date 2023/2/13
 */
@Api(tags = "主体任务类型管理")
@RestController
@RequestMapping(value = "/authsvr/v2/subTaskType")
public class AuthSubTaskTypeController {

    private IAuthSubTaskTypeFacade taskTypeFacade;

    @Autowired
    public void setTaskTypeFacade(IAuthSubTaskTypeFacade taskTypeFacade) {
        this.taskTypeFacade = taskTypeFacade;
    }


    @ApiOperation("任务类型分页查询")
    @ApiImplicitParam(name = "searchable", value = "分页查询条件")
    @PostMapping("pageSearch")
    Result<List<AuthSubTaskTypeVo>> pageSearch(SearchDTO searchDTO) {
        Page<AuthSubTaskTypeVo> voPage = taskTypeFacade.pageSearch(searchDTO);
        return Result.success(voPage.getTotalElements(), voPage.getContent());
    }

    @ApiOperation("任务类型树")
    @GetMapping("tree")
    Result<List<TaskTypeTreeVo>> taskTypeTree() {
        List<TaskTypeTreeVo> treeVos = taskTypeFacade.taskTypeTree();
        return Result.success(treeVos);
    }

    @ApiOperation("任务类型导入")
    @ApiImplicitParam(name = "fileId", value = "上传文件id")
    @GetMapping("taskTypeImp")
    ResponseStatus taskTypeImp(@RequestParam("fileId") String fileId) {
        return taskTypeFacade.taskTypeImp(fileId);
    }

    @ApiOperation(value = "任务类型同步")
    @GetMapping("taskTypeSync")
    Result<Object> taskTypeSync() {
        ResponseStatus responseStatus = taskTypeFacade.taskTypeSync();
        if (ResponseUtil.isSuccess(responseStatus)) {
            return Result.success();
        } else {
            return Result.failMessage(responseStatus.getMessage());
        }
    }


}
