package com.dragoninfo.dcuc.authweb.restcontroller.sub;

import com.dragoninfo.dcuc.app.dto.sub.DataClaAcceptDTO;
import com.dragoninfo.dcuc.app.facade.sub.IDataClaFacade;
import com.dragoninfo.dcuc.app.dto.sub.DataClaDTO;
import com.dragoninfo.dcuc.authweb.restcontroller.sub.vo.data.DataClaAcceptVo;
import com.dragoninfo.dcuc.authweb.restcontroller.sub.vo.data.DataClaVo;
import com.dragoninfo.dcuc.authweb.util.VersionUtils;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.core.search.Searchable;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author mazq
 * @date 2021/5/25
 */
@Api(tags = {"主客体授权-数据资源分类表码接口"})
@RestController
@RequestMapping(value = "/dataclassifysvr/"+ VersionUtils.VERSION_UID)
public class DataClassifyController {

    @Autowired
    private IDataClaFacade dataClaFacade;

    @ApiOperation(value = "数据资源分类分页列表")
    @ApiImplicitParam(name = "Searchable", value = "Searchable 数据资源分类分页查询对象,查询条件[name:firName op: like] [name: secName op: like] [labName op: like]")
    @PostMapping(value = "search")
    public Result dataClassifyPageList(Searchable searchable) {
        Page<DataClaAcceptDTO> page = dataClaFacade.pageSearch(searchable.toSearchDTO());
        List<DataClaAcceptDTO> content = page.getContent();
        List<DataClaAcceptVo> vos = content.stream().map(item -> {
            DataClaAcceptVo vo = new DataClaAcceptVo();
            BeanUtils.copyProperties(item, vo);
            return vo;
        }).collect(Collectors.toList());
        return Result.success(page.getTotalElements(), vos);
    }


    @ApiOperation(value = "添加数据资源分类")
    @ApiImplicitParam(name = "DataClaAcceptVo", value = "DataClaAcceptVo 数据资源分类Vo")
    @PostMapping(value = "save")
    public Result dataClassifyAdd(@RequestBody DataClaAcceptVo vo){
        DataClaAcceptDTO dto = new DataClaAcceptDTO();
        BeanUtils.copyProperties(vo,dto);
        ResponseStatus status = dataClaFacade.dataClassifyAdd(dto);
        if(status.getStatusCode().equals(ResponseStatus.SUCCESS_CODE)) {
            return Result.success(ResponseStatus.SUCCESS_CODE);
        }else {
            return Result.fail(ResponseStatus.FAIL_CODE,status.getMessage());
        }
    }

    @ApiOperation(value = "修改数据资源分类")
    @ApiImplicitParam(name = "DataClaAcceptVo", value = "DataClaAcceptVo 数据资源分类Vo")
    @PostMapping(value = "update")
    public Result dataClassifyUpdate(@RequestBody DataClaAcceptVo vo){
        DataClaAcceptDTO dto = new DataClaAcceptDTO();
        BeanUtils.copyProperties(vo,dto);
        Boolean b = dataClaFacade.dataClassifyUpdate(dto);
        if (b) {
            return Result.success();
        } else {
            return Result.fail();
        }
    }


    @ApiOperation(value = "数据资源分类详情")
    @ApiImplicitParam(name = "id", value = "id 数据资源分类 任何一级的id")
    @GetMapping(value = "detailAllLevel/{id}")
    public Result<DataClaAcceptVo> detailAllLevel(@PathVariable("id") String id){
        DataClaAcceptDTO viewDTO = dataClaFacade.detailAllLevel(id);
        DataClaAcceptVo vo = new DataClaAcceptVo();
        BeanUtils.copyProperties(viewDTO, vo);
        return Result.success(vo);
    }


    @ApiOperation(value = "根据code和path查询数据资源分类详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "code: 该级代码"),
            @ApiImplicitParam(name = "path", value = "path: 上级层级id组成 1-2-3 一级为空")
    })
    @GetMapping(value = "getByCodeAndPath")
    public Result<DataClaVo> getByCodeAndPath(@RequestParam("code") String code, @RequestParam(value = "path", required = false) String path) {
        DataClaDTO dto = dataClaFacade.getByCodeAndPath(code, path);
        DataClaVo vo = new DataClaVo();
        BeanUtils.copyProperties(dto, vo);
        return Result.success(vo);
    }

    @ApiOperation(value = "数据资源分类删除,根据Id删除所有层级数据")
    @ApiImplicitParam(name = "id", value = "数据资源分类id")
    @DeleteMapping(value = "deleteById/{id}")
    public Result deleteAllLevel(@PathVariable("id") String id){
        Boolean b = dataClaFacade.deleteAllLevel(id);
        if (b) {
            return Result.success();
        } else {
            return Result.fail();
        }
    }



}
