package com.dragoninfo.dcuc.authweb.restcontroller.sub;

import com.dragoninfo.dcuc.app.dto.sub.FieldSecDTO;
import com.dragoninfo.dcuc.app.facade.sub.IDataSecFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.sub.vo.data.DataSecVo;
import com.dragoninfo.dcuc.authweb.util.VersionUtils;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.core.search.Searchable;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author mazq
 * @date 2021/5/25
 */
@Api(tags = {"主客体授权-数据安全等级表码接口"})
@RestController
@RequestMapping(value = "/datasecsvr/"+ VersionUtils.VERSION_UID)
public class DataSecurityLevelController {

    @Autowired
    private IDataSecFacade dataSecFacade;

    @ApiOperation(value = "添加或修改数据安全等级")
    @ApiImplicitParam(name = "DataSecurityLevelVo", value = "数据安全等级Vo")
    @PostMapping(value = "save")
    public Result dataLevelAdd(@RequestBody DataSecVo vo){
        FieldSecDTO dto = new FieldSecDTO();
        BeanUtils.copyProperties(vo,dto);
        ResponseStatus status = dataSecFacade.dataLevelAdd(dto);
        if(status.getStatusCode().equals(ResponseStatus.SUCCESS_CODE)) {
            return Result.success(ResponseStatus.SUCCESS_CODE);
        }else {
            return Result.fail(ResponseStatus.FAIL_CODE,status.getMessage());
        }
    }

    @ApiOperation(value = "数据安全等级分页列表")
    @ApiImplicitParam(name = "Searchable", value = "Searchable 安全等级分页查询对象,查询条件 secName like ")
    @PostMapping(value = "search")
    public Result dataLevelPageList(Searchable searchable) {
        Page<FieldSecDTO> page = dataSecFacade.pageSearch(searchable.toSearchDTO());
        List<FieldSecDTO> content = page.getContent();
        List<DataSecVo> vos = content.stream().map(item -> {
            DataSecVo vo = new DataSecVo();
            BeanUtils.copyProperties(item, vo);
            return vo;
        }).collect(Collectors.toList());
        return Result.success(page.getTotalElements(), vos);
    }


    @ApiOperation(value = "数据安全等级详情")
    @ApiImplicitParam(name = "id", value = "id: 数据安全等级id")
    @GetMapping(value = "detail/{id}")
    public Result<DataSecVo> detail(@PathVariable("id") String id){
        FieldSecDTO dataSec = dataSecFacade.detail(id);
        DataSecVo vo = new DataSecVo();
        BeanUtils.copyProperties(dataSec, vo);
        return Result.success(vo);
    }

    @ApiOperation(value = "根据code查询数据安全等级详情")
    @ApiImplicitParam(name = "secCode", value = "secCode: 数据安全等级代码")
    @GetMapping(value = "detailByCode/{secCode}")
    public Result<DataSecVo> detailByCode(@PathVariable("secCode") String secCode){
        FieldSecDTO dataSec = dataSecFacade.detailByCode(secCode);
        DataSecVo vo = new DataSecVo();
        BeanUtils.copyProperties(dataSec, vo);
        return Result.success(vo);
    }



    @ApiOperation(value = "数据安全等级删除")
    @ApiImplicitParam(name = "id", value = "数据安全等级id")
    @DeleteMapping(value = "deleteById/{id}")
    public Result deleteById(@PathVariable("id") String id){
        Boolean b = dataSecFacade.deleteById(id);
        if (b) {
            return Result.success();
        } else {
            return Result.fail();
        }
    }



}
