package com.dragoninfo.dcuc.authweb.restcontroller.sub;

import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.app.facade.IServiceResourceFacade;
import com.dragoninfo.dcuc.app.facade.sub.IDataLevelFacade;
import com.dragoninfo.dcuc.app.facade.sub.IDataSecFacade;
import com.dragoninfo.dcuc.auth.power.facade.IAppFunInfoFacade;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthOrgInfoFacade;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthUserInfoFacade;
import com.dragoninfo.dcuc.authweb.util.VersionUtils;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 主客体授权导入相关接口
 * @author mazq
 * @date 2021/6/2
 */
@Api(tags = {"主客体授权-导入数据相关接口"})
@RestController
@RequestMapping(value = "/subimportsvr/"+ VersionUtils.VERSION_UID)
public class SubImportController {


    @Autowired
    private IApplyInfoFacade applyInfoFacade;

    @Autowired
    private IAppFunInfoFacade iAppFunInfoFacade;

    @Autowired
    private IServiceResourceFacade serviceResourceFacade;

    @Autowired
    private IAuthUserInfoFacade authUserInfoFacade;

    @Autowired
    private IAuthOrgInfoFacade authOrgInfoFacade;

    @GetMapping(value = "/app/import")
    @ApiImplicitParam(name = "fileId", value = "导入文件id")
    @ApiOperation(value = "授权主客体管理_应用资源导入")
    public Result appImpExcel(@RequestParam("fileId") String fileId) {
        ResponseStatus responseStatus = applyInfoFacade.impExcel(fileId);
        if (!responseStatus.getStatusCode().equals(ResponseStatus.SUCCESS_CODE)){
            return Result.fail(responseStatus.getStatusCode(), responseStatus.getMessage());
        }
        return Result.success();
    }

    @GetMapping(value = "fun/import")
    @ApiImplicitParam(name = "fileId", value = "导入文件id")
    @ApiOperation(value = "授权主客体管理_功能资源导入")
    public Result funImpExcel(@RequestParam("fileId") String fileId) {
        ResponseStatus responseStatus = iAppFunInfoFacade.impExcel(fileId);
        if (!responseStatus.getStatusCode().equals(ResponseStatus.SUCCESS_CODE)){
            return Result.fail(responseStatus.getStatusCode(), responseStatus.getMessage());
        }
        return Result.success();
    }


    @GetMapping(value = "service/import")
    @ApiImplicitParam(name = "fileId", value = "导入文件id")
    @ApiOperation(value = "授权主客体管理_服务资源导入")
    public Result serviceImpExcel(@RequestParam("fileId") String fileId) {
        ResponseStatus responseStatus = serviceResourceFacade.impExcel(fileId);
        if (!responseStatus.getStatusCode().equals(ResponseStatus.SUCCESS_CODE)){
            return Result.fail(responseStatus.getStatusCode(), responseStatus.getMessage());
        }
        return Result.success();
    }

    @GetMapping(value = "user/import")
    @ApiImplicitParam(name = "fileId", value = "导入文件id")
    @ApiOperation(value = "授权主客体管理_人员信息导入")
    public Result userImpExcel(@RequestParam("fileId") String fileId) {
        ResponseStatus responseStatus = authUserInfoFacade.impExcel(fileId);
        if (!responseStatus.getStatusCode().equals(ResponseStatus.SUCCESS_CODE)){
            return Result.fail(responseStatus.getStatusCode(), responseStatus.getMessage());
        }
        return Result.success();
    }

    @GetMapping(value = "org/import")
    @ApiImplicitParam(name = "fileId", value = "导入文件id")
    @ApiOperation(value = "授权主客体管理_机构信息导入")
    public Result orgImpExcel(@RequestParam("fileId") String fileId) {
        ResponseStatus responseStatus = authOrgInfoFacade.impExcel(fileId);
        if (!responseStatus.getStatusCode().equals(ResponseStatus.SUCCESS_CODE)){
            return Result.fail(responseStatus.getStatusCode(), responseStatus.getMessage());
        }
        return Result.success();
    }

}
