package com.dragoninfo.dcuc.authweb.restcontroller.sub;

import com.dragoninfo.dcuc.app.facade.IDataResourceFacade;
import com.dragoninfo.dcuc.app.facade.IResourceFacade;
import com.dragoninfo.dcuc.auth.sub.facade.IUserCenterApiFacade;
import com.dragoninfo.dcuc.authweb.util.VersionUtils;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 主客体资源同步Controller
 * @author mazq
 * @date 2021/5/27
 */
@Api(tags = {"主客体授权-同步接口"})
@RestController
@RequestMapping(value = "/syncsvr/"+ VersionUtils.VERSION_UID)
public class SubSyncController {

    @Autowired
    private IUserCenterApiFacade userCenterApiFacade;

    @Autowired
    private IResourceFacade resourceFacade;

    @Autowired
    private IDataResourceFacade dataResourceFacade;

    /**
     * 建立在机构数据已同步的基础上
     * @return
     */
    @ApiOperation(value = "授权主体-人员信息同步")
    @GetMapping(value = "userSync")
    public Result userSync() {
        ResponseStatus response = userCenterApiFacade.userSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }


    @ApiOperation(value = "授权主体-机构信息同步")
    @GetMapping(value = "orgSync")
    public Result orgSync() {
        ResponseStatus response = userCenterApiFacade.orgSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }

    @ApiOperation(value = "授权主体-人员信息信息同步")
    @GetMapping(value = "userAddSync")
    public Result userAddSync() {
        ResponseStatus response = userCenterApiFacade.userAddSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }

    @ApiOperation(value = "授权主体-人员信息信息同步")
    @GetMapping(value = "orgAddSync")
    public Result orgAddSync() {
        ResponseStatus response = userCenterApiFacade.orgAddSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }

    @ApiOperation(value = "授权主体-服务信息同步")
    @GetMapping(value = "serSync")
    public Result serSync() {
        ResponseStatus response = resourceFacade.subSerSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }

    @ApiOperation(value = "授权主体-应用信息同步")
    @GetMapping(value = "appSync")
    public Result appSync() {
        ResponseStatus response = resourceFacade.subAppSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }

    @ApiOperation(value = "授权主体-烽火应用信息同步")
    @GetMapping(value = "fhAppSync")
    public Result fhAppSync() {
        ResponseStatus response = resourceFacade.fhAppSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }

    @ApiOperation(value = "授权主体-数据分级同步")
    @GetMapping(value = "dataLevelSync")
    public Result dataLevelSync() {
        ResponseStatus response = dataResourceFacade.dataLevelSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }


    @ApiOperation(value = "授权主体-数据安全等级同步")
    @GetMapping(value = "dataSecSync")
    public Result dataSecSync() {
        ResponseStatus response = dataResourceFacade.dataSecSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }


    @ApiOperation(value = "授权主体-数据资源分类同步")
    @GetMapping(value = "dataClaSync")
    public Result dataClaSync() {
        ResponseStatus response = dataResourceFacade.dataClaSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }


    @ApiOperation(value = "授权主体-字段分类同步")
    @GetMapping(value = "fieldClaSync")
    public Result fieldClaSync() {
        ResponseStatus response = dataResourceFacade.fieldClaSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }

    @ApiOperation(value = "字段关系同步")
    @GetMapping(value = "columnRelationSync")
    public Result columnRelationSync() {
        ResponseStatus response = dataResourceFacade.columnRelationSync();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }

    @ApiOperation(value = "表和列资源同步")
    @GetMapping(value = "tableColumnSync")
    public Result tableColumnSync() {
        ResponseStatus response = dataResourceFacade.syncTableAndColumn();
        if (ResponseStatus.SUCCESS_CODE.equals(response.getStatusCode())) {
            return Result.success();
        } else {
            return Result.failMessage(response.getMessage());
        }
    }
}
